from .stores import stores
from serpapi import WalmartSearch as SerpApiSearch
from copy import deepcopy
from urllib.parse import urlsplit, parse_qsl
import json


class WalmartParams(object):
    def __init__(self, store) -> None:
        self.store = store


    @staticmethod
    def get_stores() -> list:
        return stores


    def _validate_params(self):
        store_id = next(
            (el['store_id'] for el in stores if el['store_id'] == self.store or el['address'] == self.store),
            None
        )

        if self.store and not store_id:
            raise ValueError('Please select available store (use "WalmartParams.get_stores()" to get all stores list).')

        return store_id


class WalmartSearch(WalmartParams):
    def __init__(self,
                 api_key: str,
                 query: str,
                 price_from: float = 0,
                 price_to: float = 0,
                 results_limit: int = 10,
                 sort: int = 1,
                 store = None) -> None:
        self.api_key = api_key
        self.query = query
        self.price_from = price_from
        self.price_to = price_to
        self.results_limit = results_limit
        self.sort = sort
        super().__init__(store)


    def __get_params(self) -> dict:
        params = {
            'api_key': self.api_key,        # https://serpapi.com/manage-api-key
            'engine': 'walmart',            # search engine
            'device': 'desktop',            # device to use to get the results
            'query': self.query,            # search query
            'spelling': True,               # activate spelling fix
        }

        if self.price_from != 0 or self.price_to != 0:
            if self.price_from > self.price_to or self.price_from < 0 or self.price_to < 0:
                raise ValueError('Please set correct price arguments.')
            params['min_price'] = self.price_from
            params['max_price'] = self.price_to

        if self.store:
            store_id = self._validate_params()
            params['store_id'] = store_id

        if self.sort == 1:
            params['sort'] = 'best_match'
        elif self.sort == 2:
            params['sort'] = 'best_seller'
        elif self.sort == 3:
            params['sort'] = 'price_low'
        elif self.sort == 4:
            params['sort'] = 'price_high'
        elif self.sort == 5:
            params['sort'] = 'rating_high'
        elif self.sort == 6:
            params['sort'] = 'new'
        else:
            raise ValueError('Please set correct sort argument (from 1 to 6).')

        return params


    def get_products(self) -> list:
        params = self.__get_params()        # parameters for search
        search = SerpApiSearch(params)      # data extraction on the SerpApi backend
        results = search.get_dict()         # JSON -> Python dict

        products = []
        limit = deepcopy(self.results_limit)

        # pagination if needed
        while True:
            walmart_results = results.get('organic_results', [])

            if limit < len(walmart_results):
                products.extend(walmart_results[:limit])
                break

            limit -= len(walmart_results)
            products.extend(walmart_results)

            if 'next' in results.get('serpapi_pagination', {}):
                search.params_dict.update(
                    dict(parse_qsl(urlsplit(results.get('serpapi_pagination', {}).get('next')).query)))
            else:
                break

            results = search.get_dict()     # new page results

        for index, product in enumerate(products, start=0):
            products[index]['product_id'] = deepcopy(products[index]['us_item_id'])
            products[index]['position'] = index + 1

            if products[index].get('primary_offer', {}).get('offer_id', None):
                del products[index]['primary_offer']['offer_id']
            del products[index]['us_item_id']
            del products[index]['seller_id']
            del products[index]['serpapi_product_page_url']

        return products


    def get_filters(self) -> list:
        params = self.__get_params()        # parameters for search
        search = SerpApiSearch(params)      # data extraction on the SerpApi backend
        results = search.get_dict()         # JSON -> Python dict

        return results.get('filters', 'Filters will be in the future.')


    def save_to_json(self, data) -> None:
        with open(f'walmart_search_{self.query.lower().replace(" ", "_")}.json', 'w',
                  encoding='utf-8') as json_file:
            json.dump(data, json_file, indent=2, ensure_ascii=False)


    @staticmethod
    def print(data) -> None:
        """Prints pretty JSON instead of regular print()"""
        print(json.dumps(data, indent=2, ensure_ascii=False))


class WalmartProduct(WalmartParams):
    def __init__(self,
                 api_key: str,
                 product_id,
                 reviews_limit: int = 10,
                 store = None) -> None:
        self.api_key = api_key
        self.product_id = product_id
        self.reviews_limit = reviews_limit
        super().__init__(store)


    def __get_params(self, engine) -> dict:
        # parameters for search
        params = {
            'api_key': self.api_key,        # https://serpapi.com/manage-api-key
            'engine': engine,               # search engine
            'device': 'desktop',            # device to use to get the results
            'product_id': self.product_id,  # Walmart ID of a product
        }

        if self.store and engine == 'walmart_product':
            store_id = self._validate_params()
            params['store_id'] = store_id

        return params


    def get_product_info(self) -> dict:
        params = self.__get_params(engine='walmart_product')
        search = SerpApiSearch(params)      # data extraction on the SerpApi backend
        results = search.get_dict()         # JSON -> Python dict

        product_info = results['product_result']

        product_info['product_id'] = self.product_id

        if product_info.get('arrival_date', None):
            del product_info['arrival_date']
        if product_info.get('delivery_option', {}).get('arrival_date', None):
            del product_info['delivery_option']['arrival_date']
        del product_info['us_item_id']
        del product_info['upc']
        del product_info['seller_id']
        del product_info['product_type_id']
        del product_info['offer_id']

        for index, offer in enumerate(product_info.get('offers', []), start=0):
            del product_info['offers'][index]['seller_id']
            del product_info['offers'][index]['catalog_seller_id']

        return product_info


    def get_product_reviews(self) -> list:
        params = self.__get_params(engine='walmart_product_reviews')
        params['page'] = 1                  # pagination for reviews
        search = SerpApiSearch(params)      # data extraction on the SerpApi backend
        results = search.get_dict()         # JSON -> Python dict

        reviews = []

        product_reviews = results.get('total_count', 0)

        if product_reviews:
            limit = self.reviews_limit if self.reviews_limit < product_reviews else product_reviews

            while True:
                reviews_results = results.get('reviews', [])

                if limit < len(reviews_results):
                    reviews.extend(reviews_results[:limit])
                    break

                limit -= len(reviews_results)
                reviews.extend(reviews_results)

                params['page'] += 1
                results = search.get_dict() # new page results

        # index recalculation because on each page the review position starts from 1
        for index, review in enumerate(reviews, start=0):
            review['position'] = index + 1

        return reviews


    def get_product(self) -> dict:
        product_data = {
            'product': self.get_product_info(),
            'reviews': self.get_product_reviews()
        }

        return product_data


    def save_to_json(self, data) -> None:
        with open(f'walmart_product_{self.product_id}.json', 'w', encoding='utf-8') as json_file:
            json.dump(data, json_file, indent=2, ensure_ascii=False)


    @staticmethod
    def print(data) -> None:
        """Prints pretty JSON instead of regular print()"""
        print(json.dumps(data, indent=2, ensure_ascii=False))



domains = [
  {
    "domain": "google.ad",
    "default_language_code": "ca",
    "default_country_code": "ad",
    "country_name": "Andorra",
  },
  {
    "domain": "google.ae",
    "default_language_code": "ar",
    "default_country_code": "ae",
    "country_name": "United Arab Emirates",
  },
  {
    "domain": "google.al",
    "default_language_code": "sq",
    "default_country_code": "al",
    "country_name": "Albania",
  },
  {
    "domain": "google.am",
    "default_language_code": "hy",
    "default_country_code": "am",
    "country_name": "Armenia",
  },
  {
    "domain": "google.as",
    "default_language_code": "en",
    "default_country_code": "as",
    "country_name": "American Samoa",
  },
  {
    "domain": "google.at",
    "default_language_code": "de",
    "default_country_code": "at",
    "country_name": "Austria",
  },
  {
    "domain": "google.az",
    "default_language_code": "az",
    "default_country_code": "az",
    "country_name": "Azerbaijan",
  },
  {
    "domain": "google.ba",
    "default_language_code": "bs",
    "default_country_code": "ba",
    "country_name": "Bosnia and Herzegovina",
  },
  {
    "domain": "google.be",
    "default_language_code": "nl",
    "default_country_code": "be",
    "country_name": "Belgium",
  },
  {
    "domain": "google.bf",
    "default_language_code": "fr",
    "default_country_code": "bf",
    "country_name": "Burkina Faso",
  },
  {
    "domain": "google.bg",
    "default_language_code": "bg",
    "default_country_code": "bg",
    "country_name": "Bulgaria",
  },
  {
    "domain": "google.bi",
    "default_language_code": "fr",
    "default_country_code": "bi",
    "country_name": "Burundi",
  },
  {
    "domain": "google.bj",
    "default_language_code": "fr",
    "default_country_code": "bj",
    "country_name": "Benin",
  },
  {
    "domain": "google.bs",
    "default_language_code": "en",
    "default_country_code": "bs",
    "country_name": "Bahamas",
  },
  {
    "domain": "google.bt",
    "default_language_code": "bt",
    "default_country_code": "bt",
    "country_name": "Bhutan",
  },
  {
    "domain": "google.by",
    "default_language_code": "be",
    "default_country_code": "by",
    "country_name": "Belarus",
  },
  {
    "domain": "google.ca",
    "default_language_code": "en",
    "default_country_code": "ca",
    "country_name": "Canada",
  },
  {
    "domain": "google.cd",
    "default_language_code": "fr",
    "default_country_code": "cg",
    "country_name": "Democratic Republic of the Congo",
  },
  {
    "domain": "google.cf",
    "default_language_code": "fr",
    "default_country_code": "cf",
    "country_name": "Central African Republic",
  },
  {
    "domain": "google.cg",
    "default_language_code": "fr",
    "default_country_code": "cg",
    "country_name": "Republic of the Congo",
  },
  {
    "domain": "google.ch",
    "default_language_code": "de",
    "default_country_code": "ch",
    "country_name": "Switzerland",
  },
  {
    "domain": "google.ci",
    "default_language_code": "fr",
    "default_country_code": "ci",
    "country_name": "Ivory Coast",
  },
  {
    "domain": "google.cl",
    "default_language_code": "es",
    "default_country_code": "cl",
    "country_name": "Chile",
  },
  {
    "domain": "google.cm",
    "default_language_code": "en",
    "default_country_code": "cm",
    "country_name": "Cameroon",
  },
  {
    "domain": "google.co.ao",
    "default_language_code": "pt",
    "default_country_code": "ao",
    "country_name": "Angola",
  },
  {
    "domain": "google.co.bw",
    "default_language_code": "en",
    "default_country_code": "bw",
    "country_name": "Botswana",
  },
  {
    "domain": "google.co.ck",
    "default_language_code": "en",
    "default_country_code": "ck",
    "country_name": "Cook Islands",
  },
  {
    "domain": "google.co.cr",
    "default_language_code": "es",
    "default_country_code": "cr",
    "country_name": "Costa Rica",
  },
  {
    "domain": "google.co.id",
    "default_language_code": "id",
    "default_country_code": "id",
    "country_name": "Indonesia",
  },
  {
    "domain": "google.co.il",
    "default_language_code": "iw",
    "default_country_code": "il",
    "country_name": "Israel",
  },
  {
    "domain": "google.co.in",
    "default_language_code": "hi",
    "default_country_code": "in",
    "country_name": "India",
  },
  {
    "domain": "google.co.jp",
    "default_language_code": "ja",
    "default_country_code": "jp",
    "country_name": "Japan",
  },
  {
    "domain": "google.co.ke",
    "default_language_code": "en",
    "default_country_code": "ke",
    "country_name": "Kenya",
  },
  {
    "domain": "google.co.kr",
    "default_language_code": "ko",
    "default_country_code": "kr",
    "country_name": "South Korea",
  },
  {
    "domain": "google.co.ls",
    "default_language_code": "en",
    "default_country_code": "ls",
    "country_name": "Lesotho",
  },
  {
    "domain": "google.co.ma",
    "default_language_code": "ar",
    "default_country_code": "ma",
    "country_name": "Morocco",
  },
  {
    "domain": "google.co.mz",
    "default_language_code": "pt",
    "default_country_code": "mz",
    "country_name": "Mozambique",
  },
  {
    "domain": "google.co.nz",
    "default_language_code": "en",
    "default_country_code": "nz",
    "country_name": "New Zealand",
  },
  {
    "domain": "google.co.th",
    "default_language_code": "th",
    "default_country_code": "th",
    "country_name": "Thailand",
  },
  {
    "domain": "google.co.tz",
    "default_language_code": "sw",
    "default_country_code": "tz",
    "country_name": "Tanzania",
  },
  {
    "domain": "google.co.ug",
    "default_language_code": "en",
    "default_country_code": "ug",
    "country_name": "Uganda",
  },
  {
    "domain": "google.co.uk",
    "default_language_code": "en",
    "default_country_code": "uk",
    "country_name": "United Kingdom",
  },
  {
    "domain": "google.co.uz",
    "default_language_code": "uz",
    "default_country_code": "uz",
    "country_name": "Uzbekistan",
  },
  {
    "domain": "google.co.ve",
    "default_language_code": "es",
    "default_country_code": "ve",
    "country_name": "Venezuela",
  },
  {
    "domain": "google.co.vi",
    "default_language_code": "en",
    "default_country_code": "vi",
    "country_name": "United States Virgin Islands",
  },
  {
    "domain": "google.co.za",
    "default_language_code": "af",
    "default_country_code": "za",
    "country_name": "South Africa",
  },
  {
    "domain": "google.co.zm",
    "default_language_code": "en",
    "default_country_code": "zm",
    "country_name": "Zambia",
  },
  {
    "domain": "google.co.zw",
    "default_language_code": "en",
    "default_country_code": "zw",
    "country_name": "Zimbabwe",
  },
  {
    "domain": "google.com",
    "default_language_code": "en",
    "default_country_code": "us",
    "country_name": "United States",
  },
  {
    "domain": "google.com.af",
    "default_language_code": "ps",
    "default_country_code": "af",
    "country_name": "Afghanistan",
  },
  {
    "domain": "google.com.ag",
    "default_language_code": "en",
    "default_country_code": "ag",
    "country_name": "Antigua and Barbuda",
  },
  {
    "domain": "google.com.ai",
    "default_language_code": "en",
    "default_country_code": "ai",
    "country_name": "Anguilla",
  },
  {
    "domain": "google.com.ar",
    "default_language_code": "es",
    "default_country_code": "ar",
    "country_name": "Argentina",
  },
  {
    "domain": "google.com.au",
    "default_language_code": "en",
    "default_country_code": "au",
    "country_name": "Australia",
  },
  {
    "domain": "google.com.bd",
    "default_language_code": "bn",
    "default_country_code": "bd",
    "country_name": "Bangladesh",
  },
  {
    "domain": "google.com.bh",
    "default_language_code": "ar",
    "default_country_code": "bh",
    "country_name": "Bahrain",
  },
  {
    "domain": "google.com.bn",
    "default_language_code": "my",
    "default_country_code": "bn",
    "country_name": "Brunei",
  },
  {
    "domain": "google.com.bo",
    "default_language_code": "es",
    "default_country_code": "bo",
    "country_name": "Bolivia",
  },
  {
    "domain": "google.com.br",
    "default_language_code": "pt",
    "default_country_code": "br",
    "country_name": "Brazil",
  },
  {
    "domain": "google.com.bz",
    "default_language_code": "en",
    "default_country_code": "bz",
    "country_name": "Belize",
  },
  {
    "domain": "google.com.co",
    "default_language_code": "es",
    "default_country_code": "co",
    "country_name": "Colombia",
  },
  {
    "domain": "google.com.cu",
    "default_language_code": "es",
    "default_country_code": "cu",
    "country_name": "Cuba",
  },
  {
    "domain": "google.com.cy",
    "default_language_code": "el",
    "default_country_code": "cy",
    "country_name": "Cyprus",
  },
  {
    "domain": "google.com.do",
    "default_language_code": "es",
    "default_country_code": "do",
    "country_name": "Dominican Republic",
  },
  {
    "domain": "google.com.ec",
    "default_language_code": "es",
    "default_country_code": "ec",
    "country_name": "Ecuador",
  },
  {
    "domain": "google.com.eg",
    "default_language_code": "ar",
    "default_country_code": "eg",
    "country_name": "Egypt",
  },
  {
    "domain": "google.com.et",
    "default_language_code": "am",
    "default_country_code": "et",
    "country_name": "Ethiopia",
  },
  {
    "domain": "google.com.fj",
    "default_language_code": "en",
    "default_country_code": "fj",
    "country_name": "Fiji",
  },
  {
    "domain": "google.com.gh",
    "default_language_code": "en",
    "default_country_code": "gh",
    "country_name": "Ghana",
  },
  {
    "domain": "google.com.gi",
    "default_language_code": "en",
    "default_country_code": "gi",
    "country_name": "Gibraltar",
  },
  {
    "domain": "google.com.gt",
    "default_language_code": "es",
    "default_country_code": "gt",
    "country_name": "Guatemala",
  },
  {
    "domain": "google.com.hk",
    "default_language_code": "en",
    "default_country_code": "hk",
    "country_name": "Hong Kong",
  },
  {
    "domain": "google.com.jm",
    "default_language_code": "en",
    "default_country_code": "jm",
    "country_name": "Jamaica",
  },
  {
    "domain": "google.com.kh",
    "default_language_code": "km",
    "default_country_code": "kh",
    "country_name": "Cambodia",
  },
  {
    "domain": "google.com.kw",
    "default_language_code": "ar",
    "default_country_code": "kw",
    "country_name": "Kuwait",
  },
  {
    "domain": "google.com.lb",
    "default_language_code": "ar",
    "default_country_code": "lb",
    "country_name": "Lebanon",
  },
  {
    "domain": "google.com.ly",
    "default_language_code": "ar",
    "default_country_code": "ly",
    "country_name": "Libya",
  },
  {
    "domain": "google.com.mm",
    "default_language_code": "my",
    "default_country_code": "mm",
    "country_name": "Myanmar",
  },
  {
    "domain": "google.com.mt",
    "default_language_code": "mt",
    "default_country_code": "mt",
    "country_name": "Malta",
  },
  {
    "domain": "google.com.mx",
    "default_language_code": "es",
    "default_country_code": "mx",
    "country_name": "Mexico",
  },
  {
    "domain": "google.com.my",
    "default_language_code": "my",
    "default_country_code": "my",
    "country_name": "Malaysia",
  },
  {
    "domain": "google.com.na",
    "default_language_code": "en",
    "default_country_code": "na",
    "country_name": "Namibia",
  },
  {
    "domain": "google.com.ng",
    "default_language_code": "en",
    "default_country_code": "ng",
    "country_name": "Nigeria",
  },
  {
    "domain": "google.com.ni",
    "default_language_code": "es",
    "default_country_code": "ni",
    "country_name": "Nicaragua",
  },
  {
    "domain": "google.com.np",
    "default_language_code": "ne",
    "default_country_code": "np",
    "country_name": "Nepal",
  },
  {
    "domain": "google.com.om",
    "default_language_code": "ar",
    "default_country_code": "om",
    "country_name": "Oman",
  },
  {
    "domain": "google.com.pa",
    "default_language_code": "es",
    "default_country_code": "pa",
    "country_name": "Panama",
  },
  {
    "domain": "google.com.pe",
    "default_language_code": "es",
    "default_country_code": "pe",
    "country_name": "Peru",
  },
  {
    "domain": "google.com.pg",
    "default_language_code": "en",
    "default_country_code": "pg",
    "country_name": "Papua New Guinea",
  },
  {
    "domain": "google.com.ph",
    "default_language_code": "tl",
    "default_country_code": "ph",
    "country_name": "Philippines",
  },
  {
    "domain": "google.com.pk",
    "default_language_code": "en",
    "default_country_code": "pk",
    "country_name": "Pakistan",
  },
  {
    "domain": "google.com.pr",
    "default_language_code": "es",
    "default_country_code": "pr",
    "country_name": "Puerto Rico",
  },
  {
    "domain": "google.com.py",
    "default_language_code": "es",
    "default_country_code": "py",
    "country_name": "Paraguay",
  },
  {
    "domain": "google.com.qa",
    "default_language_code": "ar",
    "default_country_code": "qa",
    "country_name": "Qatar",
  },
  {
    "domain": "google.com.sa",
    "default_language_code": "ar",
    "default_country_code": "sa",
    "country_name": "Saudi Arabia",
  },
  {
    "domain": "google.com.sb",
    "default_language_code": "en",
    "default_country_code": "sb",
    "country_name": "Solomon Islands",
  },
  {
    "domain": "google.com.sg",
    "default_language_code": "en",
    "default_country_code": "sg",
    "country_name": "Singapore",
  },
  {
    "domain": "google.com.sl",
    "default_language_code": "en",
    "default_country_code": "sl",
    "country_name": "Sierra Leone",
  },
  {
    "domain": "google.com.sv",
    "default_language_code": "es",
    "default_country_code": "sv",
    "country_name": "El Salvador",
  },
  {
    "domain": "google.com.tj",
    "default_language_code": "tg",
    "default_country_code": "tj",
    "country_name": "Tajikistan",
  },
  {
    "domain": "google.com.tr",
    "default_language_code": "tr",
    "default_country_code": "tr",
    "country_name": "Turkey",
  },
  {
    "domain": "google.com.tw",
    "default_language_code": "zh-tw",
    "default_country_code": "tw",
    "country_name": "Taiwan",
  },
  {
    "domain": "google.com.ua",
    "default_language_code": "uk",
    "default_country_code": "ua",
    "country_name": "Ukraine",
  },
  {
    "domain": "google.com.uy",
    "default_language_code": "es",
    "default_country_code": "uy",
    "country_name": "Uruguay",
  },
  {
    "domain": "google.com.vc",
    "default_language_code": "en",
    "default_country_code": "vc",
    "country_name": "Saint Vincent and the Grenadines",
  },
  {
    "domain": "google.com.vn",
    "default_language_code": "vi",
    "default_country_code": "vn",
    "country_name": "Vietnam",
  },
  {
    "domain": "google.cv",
    "default_language_code": "pt",
    "default_country_code": "cv",
    "country_name": "Cape Verde",
  },
  {
    "domain": "google.cz",
    "default_language_code": "cs",
    "default_country_code": "cz",
    "country_name": "Czech Republic",
  },
  {
    "domain": "google.de",
    "default_language_code": "de",
    "default_country_code": "de",
    "country_name": "Germany",
  },
  {
    "domain": "google.dj",
    "default_language_code": "ar",
    "default_country_code": "dj",
    "country_name": "Djibouti",
  },
  {
    "domain": "google.dk",
    "default_language_code": "da",
    "default_country_code": "dk",
    "country_name": "Denmark",
  },
  {
    "domain": "google.dm",
    "default_language_code": "en",
    "default_country_code": "dm",
    "country_name": "Dominica",
  },
  {
    "domain": "google.dz",
    "default_language_code": "ar",
    "default_country_code": "dz",
    "country_name": "Algeria",
  },
  {
    "domain": "google.ee",
    "default_language_code": "et",
    "default_country_code": "ee",
    "country_name": "Estonia",
  },
  {
    "domain": "google.es",
    "default_language_code": "es",
    "default_country_code": "es",
    "country_name": "Spain",
  },
  {
    "domain": "google.fi",
    "default_language_code": "fi",
    "default_country_code": "fi",
    "country_name": "Finland",
  },
  {
    "domain": "google.fm",
    "default_language_code": "en",
    "default_country_code": "fm",
    "country_name": "Micronesia, Federated States of",
  },
  {
    "domain": "google.fr",
    "default_language_code": "fr",
    "default_country_code": "fr",
    "country_name": "France",
  },
  {
    "domain": "google.ga",
    "default_language_code": "fr",
    "default_country_code": "ga",
    "country_name": "Gabon",
  },
  {
    "domain": "google.ge",
    "default_language_code": "ka",
    "default_country_code": "ge",
    "country_name": "Georgia",
  },
  {
    "domain": "google.gl",
    "default_language_code": "kl",
    "default_country_code": "gl",
    "country_name": "Greenland",
  },
  {
    "domain": "google.gm",
    "default_language_code": "en",
    "default_country_code": "gm",
    "country_name": "Gambia",
  },
  {
    "domain": "google.gp",
    "default_language_code": "fr",
    "default_country_code": "gp",
    "country_name": "Guadeloupe",
  },
  {
    "domain": "google.gr",
    "default_language_code": "el",
    "default_country_code": "gr",
    "country_name": "Greece",
  },
  {
    "domain": "google.gy",
    "default_language_code": "en",
    "default_country_code": "gy",
    "country_name": "Guyana",
  },
  {
    "domain": "google.hn",
    "default_language_code": "es",
    "default_country_code": "hn",
    "country_name": "Honduras",
  },
  {
    "domain": "google.hr",
    "default_language_code": "hr",
    "default_country_code": "hr",
    "country_name": "Croatia",
  },
  {
    "domain": "google.ht",
    "default_language_code": "fr",
    "default_country_code": "ht",
    "country_name": "Haiti",
  },
  {
    "domain": "google.hu",
    "default_language_code": "hu",
    "default_country_code": "hu",
    "country_name": "Hungary",
  },
  {
    "domain": "google.ie",
    "default_language_code": "en",
    "default_country_code": "ie",
    "country_name": "Ireland",
  },
  {
    "domain": "google.iq",
    "default_language_code": "ar",
    "default_country_code": "iq",
    "country_name": "Iraq",
  },
  {
    "domain": "google.is",
    "default_language_code": "is",
    "default_country_code": "is",
    "country_name": "Iceland",
  },
  {
    "domain": "google.it",
    "default_language_code": "it",
    "default_country_code": "it",
    "country_name": "Italy",
  },
  {
    "domain": "google.jo",
    "default_language_code": "ar",
    "default_country_code": "jo",
    "country_name": "Jordan",
  },
  {
    "domain": "google.kg",
    "default_language_code": "ky",
    "default_country_code": "kg",
    "country_name": "Kyrgyzstan",
  },
  {
    "domain": "google.ki",
    "default_language_code": "en",
    "default_country_code": "ki",
    "country_name": "Kiribati",
  },
  {
    "domain": "google.kz",
    "default_language_code": "kk",
    "default_country_code": "kz",
    "country_name": "Kazakhstan",
  },
  {
    "domain": "google.la",
    "default_language_code": "lo",
    "default_country_code": "la",
    "country_name": "Laos",
  },
  {
    "domain": "google.li",
    "default_language_code": "de",
    "default_country_code": "li",
    "country_name": "Liechtenstein",
  },
  {
    "domain": "google.lk",
    "default_language_code": "si",
    "default_country_code": "lk",
    "country_name": "Sri Lanka",
  },
  {
    "domain": "google.lt",
    "default_language_code": "lt",
    "default_country_code": "lt",
    "country_name": "Lithuania",
  },
  {
    "domain": "google.lu",
    "default_language_code": "fr",
    "default_country_code": "lu",
    "country_name": "Luxembourg",
  },
  {
    "domain": "google.lv",
    "default_language_code": "lv",
    "default_country_code": "lv",
    "country_name": "Latvia",
  },
  {
    "domain": "google.md",
    "default_language_code": "ro",
    "default_country_code": "md",
    "country_name": "Moldova",
  },
  {
    "domain": "google.mg",
    "default_language_code": "fr",
    "default_country_code": "mg",
    "country_name": "Madagascar",
  },
  {
    "domain": "google.mk",
    "default_language_code": "mk",
    "default_country_code": "mk",
    "country_name": "Macedonia",
  },
  {
    "domain": "google.ml",
    "default_language_code": "fr",
    "default_country_code": "ml",
    "country_name": "Mali",
  },
  {
    "domain": "google.mn",
    "default_language_code": "mn",
    "default_country_code": "mn",
    "country_name": "Mongolia",
  },
  {
    "domain": "google.ms",
    "default_language_code": "en",
    "default_country_code": "ms",
    "country_name": "Montserrat",
  },
  {
    "domain": "google.mu",
    "default_language_code": "en",
    "default_country_code": "mu",
    "country_name": "Mauritius",
  },
  {
    "domain": "google.mv",
    "default_language_code": "mv",
    "default_country_code": "mv",
    "country_name": "Maldives",
  },
  {
    "domain": "google.mw",
    "default_language_code": "en",
    "default_country_code": "mw",
    "country_name": "Malawi",
  },
  {
    "domain": "google.ne",
    "default_language_code": "fr",
    "default_country_code": "ne",
    "country_name": "Niger",
  },
  {
    "domain": "google.nl",
    "default_language_code": "nl",
    "default_country_code": "nl",
    "country_name": "Netherlands",
  },
  {
    "domain": "google.no",
    "default_language_code": "no",
    "default_country_code": "no",
    "country_name": "Norway",
  },
  {
    "domain": "google.nr",
    "default_language_code": "en",
    "default_country_code": "nr",
    "country_name": "Nauru",
  },
  {
    "domain": "google.nu",
    "default_language_code": "en",
    "default_country_code": "nu",
    "country_name": "Niue",
  },
  {
    "domain": "google.pl",
    "default_language_code": "pl",
    "default_country_code": "pl",
    "country_name": "Poland",
  },
  {
    "domain": "google.ps",
    "default_language_code": "ar",
    "default_country_code": "ps",
    "country_name": "Palestine",
  },
  {
    "domain": "google.pt",
    "default_language_code": "pt",
    "default_country_code": "pt",
    "country_name": "Portugal",
  },
  {
    "domain": "google.ro",
    "default_language_code": "ro",
    "default_country_code": "ro",
    "country_name": "Romania",
  },
  {
    "domain": "google.rs",
    "default_language_code": "sr",
    "default_country_code": "rs",
    "country_name": "Serbia",
  },
  {
    "domain": "google.ru",
    "default_language_code": "ru",
    "default_country_code": "ru",
    "country_name": "Russia",
  },
  {
    "domain": "google.rw",
    "default_language_code": "rw",
    "default_country_code": "rw",
    "country_name": "Rwanda",
  },
  {
    "domain": "google.sc",
    "default_language_code": "fr",
    "default_country_code": "sc",
    "country_name": "Seychelles",
  },
  {
    "domain": "google.se",
    "default_language_code": "sv",
    "default_country_code": "se",
    "country_name": "Sweden",
  },
  {
    "domain": "google.sh",
    "default_language_code": "en",
    "default_country_code": "sh",
    "country_name": "Saint Helena, Ascension and Tristan da Cunha",
  },
  {
    "domain": "google.si",
    "default_language_code": "sl",
    "default_country_code": "si",
    "country_name": "Slovenia",
  },
  {
    "domain": "google.sk",
    "default_language_code": "sk",
    "default_country_code": "sk",
    "country_name": "Slovakia",
  },
  {
    "domain": "google.sm",
    "default_language_code": "it",
    "default_country_code": "sm",
    "country_name": "San Marino",
  },
  {
    "domain": "google.sn",
    "default_language_code": "fr",
    "default_country_code": "sn",
    "country_name": "Senegal",
  },
  {
    "domain": "google.so",
    "default_language_code": "so",
    "default_country_code": "so",
    "country_name": "Somalia",
  },
  {
    "domain": "google.sr",
    "default_language_code": "nl",
    "default_country_code": "sr",
    "country_name": "Suriname",
  },
  {
    "domain": "google.td",
    "default_language_code": "ar",
    "default_country_code": "td",
    "country_name": "Chad",
  },
  {
    "domain": "google.tg",
    "default_language_code": "fr",
    "default_country_code": "tg",
    "country_name": "Togo",
  },
  {
    "domain": "google.tk",
    "default_language_code": "en",
    "default_country_code": "tk",
    "country_name": "Tokelau",
  },
  {
    "domain": "google.tl",
    "default_language_code": "pt",
    "default_country_code": "tl",
    "country_name": "Timor-Leste",
  },
  {
    "domain": "google.tm",
    "default_language_code": "tk",
    "default_country_code": "tm",
    "country_name": "Turkmenistan",
  },
  {
    "domain": "google.tn",
    "default_language_code": "ar",
    "default_country_code": "tn",
    "country_name": "Tunisia",
  },
  {
    "domain": "google.to",
    "default_language_code": "en",
    "default_country_code": "to",
    "country_name": "Tonga",
  },
  {
    "domain": "google.tt",
    "default_language_code": "en",
    "default_country_code": "tt",
    "country_name": "Trinidad and Tobago",
  },
  {
    "domain": "google.vg",
    "default_language_code": "en",
    "default_country_code": "vg",
    "country_name": "British Virgin Islands",
  },
  {
    "domain": "google.vu",
    "default_language_code": "vu",
    "default_country_code": "vu",
    "country_name": "Vanuatu",
  },
  {
    "domain": "google.ws",
    "default_language_code": "ws",
    "default_country_code": "ws",
    "country_name": "Samoa",
  },
]


from .domains import domains
from selenium import webdriver
from selenium_stealth import stealth
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selectolax.lexbor import LexborHTMLParser
from serpapi import EbaySearch as SerpApiSearch
from copy import deepcopy
from urllib.parse import urlsplit, parse_qsl
import json, re, time


class EbayParams(object):
    def __init__(self, domain) -> None:
        self.domain = domain


    @staticmethod
    def get_domains() -> list:
        return domains


    def _validate_params(self):
        selected_domain = next(
            ((el['domain'] for el in domains if el['domain'] == self.domain or el['country'] == self.domain)),
            None
        )

        if self.domain and not selected_domain:
            raise ValueError('Please select available domain (use "EbayParams.get_domains()" to get all domains list).')

        return selected_domain


class EbaySearch(EbayParams):
    def __init__(self,
                 api_key: str,
                 query: str,
                 price_from: float = 0,
                 price_to: float = 0,
                 results_limit: int = 10,
                 domain = 'ebay.com') -> None:
        self.api_key = api_key
        self.query = query
        self.price_from = price_from
        self.price_to = price_to
        self.results_limit = results_limit
        super().__init__(domain)


    def __get_params(self) -> dict:
        domain = self._validate_params()

        params = {
            'api_key': self.api_key,        # https://serpapi.com/manage-api-key
            'engine': 'ebay',            # search engine
            'device': 'desktop',            # device to use to get the results
            'ebay_domain': domain,          # eBay domain to use
            '_nkw': self.query,             # search query
            '_ipg': 200,                    # number of items per page
            '_pgn': 1,                      # pagination
        }

        if self.price_from != 0 or self.price_to != 0:
            if self.price_from > self.price_to or self.price_from < 0 or self.price_to < 0:
                raise ValueError('Please set correct price arguments.')
            params['_udlo'] = self.price_from
            params['_udhi'] = self.price_to

        return params


    def get_products(self) -> list:
        params = self.__get_params()        # parameters for search
        search = SerpApiSearch(params)      # data extraction on the SerpApi backend
        results = search.get_dict()         # JSON -> Python dict

        products = []
        limit = deepcopy(self.results_limit)

        # pagination if needed
        while 'error' not in results:
            ebay_results = results.get('organic_results', [])

            if limit < len(ebay_results):
                products.extend(ebay_results[:limit])
                break

            limit -= len(ebay_results)
            products.extend(ebay_results)

            params['_pgn'] += 1
            results = search.get_dict()     # new page results

        for index, product in enumerate(products, start=0):
            products[index]['position'] = index + 1

        return products


    def save_to_json(self, data) -> None:
        with open(f'ebay_search_{self.query.lower().replace(" ", "_")}.json', 'w',
                  encoding='utf-8') as json_file:
            json.dump(data, json_file, indent=2, ensure_ascii=False)


    @staticmethod
    def print(data) -> None:
        """Prints pretty JSON instead of regular print()"""
        print(json.dumps(data, indent=2, ensure_ascii=False))


class EbayProduct(object):
    def __init__(self, link: str, reviews_limit: int = 10, multiplier: float = 1.0) -> None:
        self.link = link
        self.reviews_limit = reviews_limit
        self.multiplier = multiplier
        self.driver = self.__get_browser_instance()
        self.product_id = self.__get_product_id()


    def __del__(self):
        self.driver.quit()


    def __get_product_id(self):
        # https://regex101.com/r/82v2sq/1
        product_id = re.search(r'/(\d{7,})\??', self.link).group(1)

        return product_id


    def __get_browser_instance(self):
        service = Service(ChromeDriverManager().install())

        options = webdriver.ChromeOptions()
        options.add_argument('start-maximized')
        options.add_argument('--headless')
        options.add_argument('--no-sandbox')
        options.add_experimental_option('excludeSwitches', ['enable-automation', 'enable-logging'])
        options.add_experimental_option('useAutomationExtension', False)

        driver = webdriver.Chrome(service=service, options=options)

        stealth(
            driver,
            user_agent='Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36',
            languages=['en-US', 'en'],
            vendor='Google Inc.',
            platform='Win32',
            webgl_vendor='Intel Inc.',
            renderer='Intel Iris OpenGL Engine',
            fix_hairline=True,
        )

        driver.get(self.link)

        return driver


    def get_product_info(self) -> dict:
        parser = LexborHTMLParser(self.driver.page_source)

        title = parser.root.css_first('[data-testid="x-item-title"]').text(strip=True)

        raw_condition = parser.root.css_first('[data-testid="x-item-condition"] [data-testid="ux-textual-display"]')
        condition = raw_condition.text(strip=True) if raw_condition else None

        raw_price = parser.root.css_first('[data-testid="x-price-primary"] > span > span')
        price = raw_price.text(strip=True) if raw_price else None

        item_specifics = {}
        item_specifics_elements = parser.root.css('[data-testid="x-about-this-item"] [data-testid="ux-layout-section__item"], [data-testid="x-about-this-item"] [data-testid="ux-layout-section-evo__item"]')

        for element in item_specifics_elements:
            label = element.css_first('.ux-labels-values__labels:nth-child(1) span').text(strip=True)
            value = element.css_first('.ux-labels-values__values:nth-child(2) span').text(strip=True)
            item_specifics[label] = value

            if element.css_first('.ux-labels-values__labels:nth-child(3)'):
                sub_label = element.css_first('.ux-labels-values__labels:nth-child(3) span').text(strip=True)
                sub_value = element.css_first('.ux-labels-values__values:nth-child(4) span').text(strip=True)
                item_specifics[sub_label] = sub_value

        about_product = {}
        about_product_elements = parser.root.css('[data-testid="x-product-details"] [data-testid="ux-layout-section"], [data-testid="x-product-details"] [data-testid="ux-layout-section-module-evo"]')

        for element in about_product_elements:
            section_title = element.css_first('.section-title').text(strip=True)
            labels_values = {}
            labels_values_elements = element.css('[data-testid="ux-labels-values"]')

            for lv_element in labels_values_elements:
                lv_label = lv_element.css_first('.ux-labels-values__labels-content').text(strip=True)
                lv_value = lv_element.css_first('.ux-labels-values__values-content').text(strip=True)
                labels_values[lv_label] = lv_value

            about_product[section_title] = labels_values

        raw_rating = parser.root.css_first('#review-ratings-cntr > span:first-child')
        rating = float(raw_rating.attributes.get('title').split(' ')[0]) if raw_rating else None

        raw_reviews = parser.root.css_first('#review-ratings-cntr > span:last-child')
        reviews = int(raw_reviews.text(strip=True).split(' ')[0].replace(',', '').replace('.', '')) if raw_reviews else None

        description = ''
        frames = self.driver.find_elements(By.TAG_NAME, "iframe")

        for frame in frames:
            self.driver.switch_to.frame(frame)

            try:
                text = self.driver.find_element(By.CSS_SELECTOR, "#ds_div").text.strip()
                if text:
                    description = text
                    self.driver.switch_to.default_content()
                    break
            except:
                pass

            self.driver.switch_to.default_content()

        data = {
            'title': title,
            'link': self.link,
            'product_id': self.product_id,
            'condition': condition,
            'price': price,
            'item_specifics': item_specifics,
            'about_product': about_product,
            'description': description,
            'rating': rating,
            'reviews': reviews
        }

        return data


    def get_product_reviews(self) -> list:
        reviews = []

        try:
            self.driver.execute_script('document.querySelector(".reviews-right .reviews-header a").click()')
            time.sleep(self.multiplier * 1)

            while True:
                self.driver.find_element(By.CSS_SELECTOR, 'body').send_keys(Keys.END)
                time.sleep(self.multiplier * 1)

                parser = LexborHTMLParser(self.driver.page_source)

                for el in parser.css('.ebay-review-section'):
                    name = el.css_first('[itemprop="author"]').text(strip=True)
                    raw_rating = el.css_first('[itemprop="reviewRating"]')
                    rating = float(raw_rating.attributes.get('title', '0 ').split(' ')[0]) if raw_rating else None
                    raw_heading = el.css_first('[itemprop="name"]')
                    heading = raw_heading.text(strip=True) if raw_heading else None
                    date = el.css_first('[itemprop="datePublished"]').text(strip=True)
                    raw_badge = el.css_first('.ebay-review-section-l > span:nth-child(5)')
                    badge = raw_badge.text(strip=True) if raw_badge else None
                    raw_review = el.css_first('[itemprop="reviewBody"]')
                    review = raw_review.text(strip=True) if raw_review else None

                    reviews.append({
                        'name': name,
                        'rating': rating,
                        'heading': heading,
                        'date': date,
                        'badge': badge,
                        'review': review
                    })

                if len(reviews) >= self.reviews_limit:
                    reviews = reviews[:self.reviews_limit]
                    break

                try:
                    self.driver.find_element(By.CSS_SELECTOR, 'nav li:last-child > a:not(.disabled)').click()
                    time.sleep(self.multiplier * 1)
                    self.driver.find_element(By.CSS_SELECTOR, '.ebay-review-section')
                except:
                    break

            self.driver.find_element(By.CSS_SELECTOR, '.vertical-align a').click()
            time.sleep(self.multiplier * 1)

            for index, review in enumerate(reviews, start=0):
                reviews[index]['position'] = index + 1
        except:
            pass
        finally:
            return reviews


    def get_product(self) -> dict:
        product_data = {
            'product': self.get_product_info(),
            'reviews': self.get_product_reviews()
        }

        return product_data


    def save_to_json(self, data) -> None:
        with open(f'ebay_product_{self.product_id}.json', 'w', encoding='utf-8') as json_file:
            json.dump(data, json_file, indent=2, ensure_ascii=False)


    @staticmethod
    def print(data) -> None:
        """Prints pretty JSON instead of regular print()"""
        print(json.dumps(data, indent=2, ensure_ascii=False))



countries = [
  {
    "code": "af",
    "name": "Afghanistan",
  },
  {
    "code": "al",
    "name": "Albania",
  },
  {
    "code": "dz",
    "name": "Algeria",
  },
  {
    "code": "as",
    "name": "American Samoa",
  },
  {
    "code": "ad",
    "name": "Andorra",
  },
  {
    "code": "ao",
    "name": "Angola",
  },
  {
    "code": "ai",
    "name": "Anguilla",
  },
  {
    "code": "aq",
    "name": "Antarctica",
  },
  {
    "code": "ag",
    "name": "Antigua and Barbuda",
  },
  {
    "code": "ar",
    "name": "Argentina",
  },
  {
    "code": "am",
    "name": "Armenia",
  },
  {
    "code": "aw",
    "name": "Aruba",
  },
  {
    "code": "au",
    "name": "Australia",
  },
  {
    "code": "at",
    "name": "Austria",
  },
  {
    "code": "az",
    "name": "Azerbaijan",
  },
  {
    "code": "bs",
    "name": "Bahamas",
  },
  {
    "code": "bh",
    "name": "Bahrain",
  },
  {
    "code": "bd",
    "name": "Bangladesh",
  },
  {
    "code": "bb",
    "name": "Barbados",
  },
  {
    "code": "by",
    "name": "Belarus",
  },
  {
    "code": "be",
    "name": "Belgium",
  },
  {
    "code": "bz",
    "name": "Belize",
  },
  {
    "code": "bj",
    "name": "Benin",
  },
  {
    "code": "bm",
    "name": "Bermuda",
  },
  {
    "code": "bt",
    "name": "Bhutan",
  },
  {
    "code": "bo",
    "name": "Bolivia",
  },
  {
    "code": "ba",
    "name": "Bosnia and Herzegovina",
  },
  {
    "code": "bw",
    "name": "Botswana",
  },
  {
    "code": "bv",
    "name": "Bouvet Island",
  },
  {
    "code": "br",
    "name": "Brazil",
  },
  {
    "code": "io",
    "name": "British Indian Ocean Territory",
  },
  {
    "code": "bn",
    "name": "Brunei Darussalam",
  },
  {
    "code": "bg",
    "name": "Bulgaria",
  },
  {
    "code": "bf",
    "name": "Burkina Faso",
  },
  {
    "code": "bi",
    "name": "Burundi",
  },
  {
    "code": "kh",
    "name": "Cambodia",
  },
  {
    "code": "cm",
    "name": "Cameroon",
  },
  {
    "code": "ca",
    "name": "Canada",
  },
  {
    "code": "cv",
    "name": "Cape Verde",
  },
  {
    "code": "ky",
    "name": "Cayman Islands",
  },
  {
    "code": "cf",
    "name": "Central African Republic",
  },
  {
    "code": "td",
    "name": "Chad",
  },
  {
    "code": "cl",
    "name": "Chile",
  },
  {
    "code": "cn",
    "name": "China",
  },
  {
    "code": "cx",
    "name": "Christmas Island",
  },
  {
    "code": "cc",
    "name": "Cocos (Keeling) Islands",
  },
  {
    "code": "co",
    "name": "Colombia",
  },
  {
    "code": "km",
    "name": "Comoros",
  },
  {
    "code": "cg",
    "name": "Congo",
  },
  {
    "code": "cd",
    "name": "Congo, the Democratic Republic of the",
  },
  {
    "code": "ck",
    "name": "Cook Islands",
  },
  {
    "code": "cr",
    "name": "Costa Rica",
  },
  {
    "code": "ci",
    "name": "Cote D'ivoire",
  },
  {
    "code": "hr",
    "name": "Croatia",
  },
  {
    "code": "cu",
    "name": "Cuba",
  },
  {
    "code": "cy",
    "name": "Cyprus",
  },
  {
    "code": "cz",
    "name": "Czech Republic",
  },
  {
    "code": "dk",
    "name": "Denmark",
  },
  {
    "code": "dj",
    "name": "Djibouti",
  },
  {
    "code": "dm",
    "name": "Dominica",
  },
  {
    "code": "do",
    "name": "Dominican Republic",
  },
  {
    "code": "ec",
    "name": "Ecuador",
  },
  {
    "code": "eg",
    "name": "Egypt",
  },
  {
    "code": "sv",
    "name": "El Salvador",
  },
  {
    "code": "gq",
    "name": "Equatorial Guinea",
  },
  {
    "code": "er",
    "name": "Eritrea",
  },
  {
    "code": "ee",
    "name": "Estonia",
  },
  {
    "code": "et",
    "name": "Ethiopia",
  },
  {
    "code": "fk",
    "name": "Falkland Islands (Malvinas)",
  },
  {
    "code": "fo",
    "name": "Faroe Islands",
  },
  {
    "code": "fj",
    "name": "Fiji",
  },
  {
    "code": "fi",
    "name": "Finland",
  },
  {
    "code": "fr",
    "name": "France",
  },
  {
    "code": "gf",
    "name": "French Guiana",
  },
  {
    "code": "pf",
    "name": "French Polynesia",
  },
  {
    "code": "tf",
    "name": "French Southern Territories",
  },
  {
    "code": "ga",
    "name": "Gabon",
  },
  {
    "code": "gm",
    "name": "Gambia",
  },
  {
    "code": "ge",
    "name": "Georgia",
  },
  {
    "code": "de",
    "name": "Germany",
  },
  {
    "code": "gh",
    "name": "Ghana",
  },
  {
    "code": "gi",
    "name": "Gibraltar",
  },
  {
    "code": "gr",
    "name": "Greece",
  },
  {
    "code": "gl",
    "name": "Greenland",
  },
  {
    "code": "gd",
    "name": "Grenada",
  },
  {
    "code": "gp",
    "name": "Guadeloupe",
  },
  {
    "code": "gu",
    "name": "Guam",
  },
  {
    "code": "gt",
    "name": "Guatemala",
  },
  {
    "code": "gn",
    "name": "Guinea",
  },
  {
    "code": "gw",
    "name": "Guinea-Bissau",
  },
  {
    "code": "gy",
    "name": "Guyana",
  },
  {
    "code": "ht",
    "name": "Haiti",
  },
  {
    "code": "hm",
    "name": "Heard Island and Mcdonald Islands",
  },
  {
    "code": "va",
    "name": "Holy See (Vatican City State)",
  },
  {
    "code": "hn",
    "name": "Honduras",
  },
  {
    "code": "hk",
    "name": "Hong Kong",
  },
  {
    "code": "hu",
    "name": "Hungary",
  },
  {
    "code": "is",
    "name": "Iceland",
  },
  {
    "code": "in",
    "name": "India",
  },
  {
    "code": "id",
    "name": "Indonesia",
  },
  {
    "code": "ir",
    "name": "Iran, Islamic Republic of",
  },
  {
    "code": "iq",
    "name": "Iraq",
  },
  {
    "code": "ie",
    "name": "Ireland",
  },
  {
    "code": "il",
    "name": "Israel",
  },
  {
    "code": "it",
    "name": "Italy",
  },
  {
    "code": "jm",
    "name": "Jamaica",
  },
  {
    "code": "jp",
    "name": "Japan",
  },
  {
    "code": "jo",
    "name": "Jordan",
  },
  {
    "code": "kz",
    "name": "Kazakhstan",
  },
  {
    "code": "ke",
    "name": "Kenya",
  },
  {
    "code": "ki",
    "name": "Kiribati",
  },
  {
    "code": "kp",
    "name": "Korea, Democratic People's Republic of",
  },
  {
    "code": "kr",
    "name": "Korea, Republic of",
  },
  {
    "code": "kw",
    "name": "Kuwait",
  },
  {
    "code": "kg",
    "name": "Kyrgyzstan",
  },
  {
    "code": "la",
    "name": "Lao People's Democratic Republic",
  },
  {
    "code": "lv",
    "name": "Latvia",
  },
  {
    "code": "lb",
    "name": "Lebanon",
  },
  {
    "code": "ls",
    "name": "Lesotho",
  },
  {
    "code": "lr",
    "name": "Liberia",
  },
  {
    "code": "ly",
    "name": "Libyan Arab Jamahiriya",
  },
  {
    "code": "li",
    "name": "Liechtenstein",
  },
  {
    "code": "lt",
    "name": "Lithuania",
  },
  {
    "code": "lu",
    "name": "Luxembourg",
  },
  {
    "code": "mo",
    "name": "Macao",
  },
  {
    "code": "mk",
    "name": "Macedonia, the Former Yugosalv Republic of",
  },
  {
    "code": "mg",
    "name": "Madagascar",
  },
  {
    "code": "mw",
    "name": "Malawi",
  },
  {
    "code": "my",
    "name": "Malaysia",
  },
  {
    "code": "mv",
    "name": "Maldives",
  },
  {
    "code": "ml",
    "name": "Mali",
  },
  {
    "code": "mt",
    "name": "Malta",
  },
  {
    "code": "mh",
    "name": "Marshall Islands",
  },
  {
    "code": "mq",
    "name": "Martinique",
  },
  {
    "code": "mr",
    "name": "Mauritania",
  },
  {
    "code": "mu",
    "name": "Mauritius",
  },
  {
    "code": "yt",
    "name": "Mayotte",
  },
  {
    "code": "mx",
    "name": "Mexico",
  },
  {
    "code": "fm",
    "name": "Micronesia, Federated States of",
  },
  {
    "code": "md",
    "name": "Moldova, Republic of",
  },
  {
    "code": "mc",
    "name": "Monaco",
  },
  {
    "code": "mn",
    "name": "Mongolia",
  },
  {
    "code": "ms",
    "name": "Montserrat",
  },
  {
    "code": "ma",
    "name": "Morocco",
  },
  {
    "code": "mz",
    "name": "Mozambique",
  },
  {
    "code": "mm",
    "name": "Myanmar",
  },
  {
    "code": "na",
    "name": "Namibia",
  },
  {
    "code": "nr",
    "name": "Nauru",
  },
  {
    "code": "np",
    "name": "Nepal",
  },
  {
    "code": "nl",
    "name": "Netherlands",
  },
  {
    "code": "an",
    "name": "Netherlands Antilles",
  },
  {
    "code": "nc",
    "name": "New Caledonia",
  },
  {
    "code": "nz",
    "name": "New Zealand",
  },
  {
    "code": "ni",
    "name": "Nicaragua",
  },
  {
    "code": "ne",
    "name": "Niger",
  },
  {
    "code": "ng",
    "name": "Nigeria",
  },
  {
    "code": "nu",
    "name": "Niue",
  },
  {
    "code": "nf",
    "name": "Norfolk Island",
  },
  {
    "code": "mp",
    "name": "Northern Mariana Islands",
  },
  {
    "code": "no",
    "name": "Norway",
  },
  {
    "code": "om",
    "name": "Oman",
  },
  {
    "code": "pk",
    "name": "Pakistan",
  },
  {
    "code": "pw",
    "name": "Palau",
  },
  {
    "code": "ps",
    "name": "Palestinian Territory, Occupied",
  },
  {
    "code": "pa",
    "name": "Panama",
  },
  {
    "code": "pg",
    "name": "Papua New Guinea",
  },
  {
    "code": "py",
    "name": "Paraguay",
  },
  {
    "code": "pe",
    "name": "Peru",
  },
  {
    "code": "ph",
    "name": "Philippines",
  },
  {
    "code": "pn",
    "name": "Pitcairn",
  },
  {
    "code": "pl",
    "name": "Poland",
  },
  {
    "code": "pt",
    "name": "Portugal",
  },
  {
    "code": "pr",
    "name": "Puerto Rico",
  },
  {
    "code": "qa",
    "name": "Qatar",
  },
  {
    "code": "re",
    "name": "Reunion",
  },
  {
    "code": "ro",
    "name": "Romania",
  },
  {
    "code": "ru",
    "name": "Russian Federation",
  },
  {
    "code": "rw",
    "name": "Rwanda",
  },
  {
    "code": "sh",
    "name": "Saint Helena",
  },
  {
    "code": "kn",
    "name": "Saint Kitts and Nevis",
  },
  {
    "code": "lc",
    "name": "Saint Lucia",
  },
  {
    "code": "pm",
    "name": "Saint Pierre and Miquelon",
  },
  {
    "code": "vc",
    "name": "Saint Vincent and the Grenadines",
  },
  {
    "code": "ws",
    "name": "Samoa",
  },
  {
    "code": "sm",
    "name": "San Marino",
  },
  {
    "code": "st",
    "name": "Sao Tome and Principe",
  },
  {
    "code": "sa",
    "name": "Saudi Arabia",
  },
  {
    "code": "sn",
    "name": "Senegal",
  },
  {
    "code": "rs",
    "name": "Serbia and Montenegro",
  },
  {
    "code": "sc",
    "name": "Seychelles",
  },
  {
    "code": "sl",
    "name": "Sierra Leone",
  },
  {
    "code": "sg",
    "name": "Singapore",
  },
  {
    "code": "sk",
    "name": "Slovakia",
  },
  {
    "code": "si",
    "name": "Slovenia",
  },
  {
    "code": "sb",
    "name": "Solomon Islands",
  },
  {
    "code": "so",
    "name": "Somalia",
  },
  {
    "code": "za",
    "name": "South Africa",
  },
  {
    "code": "gs",
    "name": "South Georgia and the South Sandwich Islands",
  },
  {
    "code": "es",
    "name": "Spain",
  },
  {
    "code": "lk",
    "name": "Sri Lanka",
  },
  {
    "code": "sd",
    "name": "Sudan",
  },
  {
    "code": "sr",
    "name": "Suriname",
  },
  {
    "code": "sj",
    "name": "Svalbard and Jan Mayen",
  },
  {
    "code": "sz",
    "name": "Swaziland",
  },
  {
    "code": "se",
    "name": "Sweden",
  },
  {
    "code": "ch",
    "name": "Switzerland",
  },
  {
    "code": "sy",
    "name": "Syrian Arab Republic",
  },
  {
    "code": "tw",
    "name": "Taiwan, Province of China",
  },
  {
    "code": "tj",
    "name": "Tajikistan",
  },
  {
    "code": "tz",
    "name": "Tanzania, United Republic of",
  },
  {
    "code": "th",
    "name": "Thailand",
  },
  {
    "code": "tl",
    "name": "Timor-Leste",
  },
  {
    "code": "tg",
    "name": "Togo",
  },
  {
    "code": "tk",
    "name": "Tokelau",
  },
  {
    "code": "to",
    "name": "Tonga",
  },
  {
    "code": "tt",
    "name": "Trinidad and Tobago",
  },
  {
    "code": "tn",
    "name": "Tunisia",
  },
  {
    "code": "tr",
    "name": "Turkey",
  },
  {
    "code": "tm",
    "name": "Turkmenistan",
  },
  {
    "code": "tc",
    "name": "Turks and Caicos Islands",
  },
  {
    "code": "tv",
    "name": "Tuvalu",
  },
  {
    "code": "ug",
    "name": "Uganda",
  },
  {
    "code": "ua",
    "name": "Ukraine",
  },
  {
    "code": "ae",
    "name": "United Arab Emirates",
  },
  {
    "code": "uk",
    "name": "United Kingdom",
  },
  {
    "code": "gb",
    "name": "United Kingdom",
  },
  {
    "code": "us",
    "name": "United States",
  },
  {
    "code": "um",
    "name": "United States Minor Outlying Islands",
  },
  {
    "code": "uy",
    "name": "Uruguay",
  },
  {
    "code": "uz",
    "name": "Uzbekistan",
  },
  {
    "code": "vu",
    "name": "Vanuatu",
  },
  {
    "code": "ve",
    "name": "Venezuela",
  },
  {
    "code": "vn",
    "name": "Viet Nam",
  },
  {
    "code": "vg",
    "name": "Virgin Islands, British",
  },
  {
    "code": "vi",
    "name": "Virgin Islands, U.S.",
  },
  {
    "code": "wf",
    "name": "Wallis and Futuna",
  },
  {
    "code": "eh",
    "name": "Western Sahara",
  },
  {
    "code": "ye",
    "name": "Yemen",
  },
  {
    "code": "zm",
    "name": "Zambia",
  },
  {
    "code": "zw",
    "name": "Zimbabwe",
  },
]


from serpapi import HomeDepotSearch as SerpApiSearch
from copy import deepcopy
from urllib.parse import urlsplit, parse_qsl
import json, re


class HomeDepotSearch(object):
    def __init__(self,
                 api_key: str,
                 query: str,
                 price_from: float = 0,
                 price_to: float = 0,
                 results_limit: int = 10,
                 sort: int = 1,
                 zip_code = None) -> None:
        self.api_key = api_key
        self.query = query
        self.price_from = price_from
        self.price_to = price_to
        self.results_limit = results_limit
        self.sort = sort
        self.zip_code = zip_code


    def __get_params(self) -> dict:
        params = {
            'api_key': self.api_key,        # https://serpapi.com/manage-api-key
            'engine': 'home_depot',         # search engine
            'device': 'desktop',            # device to use to get the results
            'q': self.query,                # search query
            'ps': 48,                       # number of items per page
            'page': 1,                      # pagination
        }

        # https://regex101.com/r/UnEKoe/1
        zip_pattern = re.compile(r'^\d{5}(-\d{4})?$')

        if self.zip_code:
            if not zip_pattern.match(self.zip_code):
                raise ValueError('Please set a correct ZIP code.')
            params['delivery_zip'] = self.zip_code

        if self.price_from != 0 or self.price_to != 0:
            if self.price_from > self.price_to or self.price_from < 0 or self.price_to < 0:
                raise ValueError('Please set correct price arguments.')
            params['lowerbound'] = self.price_from
            params['upperbound'] = self.price_to

        if self.sort == 1:
            params['hd_sort'] = 'best_match'
        elif self.sort == 2:
            params['hd_sort'] = 'top_sellers'
        elif self.sort == 3:
            params['hd_sort'] = 'price_low_to_high'
        elif self.sort == 4:
            params['hd_sort'] = 'price_high_to_low'
        elif self.sort == 5:
            params['hd_sort'] = 'top_rated'
        else:
            raise ValueError('Please set correct sort argument (from 1 to 5).')

        return params


    def get_products(self) -> list:
        params = self.__get_params()        # parameters for search
        search = SerpApiSearch(params)      # data extraction on the SerpApi backend
        results = search.get_dict()         # JSON -> Python dict

        products = []
        limit = deepcopy(self.results_limit)

        # pagination if needed
        while 'error' not in results:
            home_depot_results = results.get('products', [])

            if limit < len(home_depot_results):
                products.extend(home_depot_results[:limit])
                break

            limit -= len(home_depot_results)
            products.extend(home_depot_results)

            params['page'] += 1
            results = search.get_dict()     # new page results

        for index, product in enumerate(products, start=0):
            products[index]['position'] = index + 1

            del products[index]['serpapi_link']

        return products


    def get_filters(self) -> list:
        params = self.__get_params()        # parameters for search
        search = SerpApiSearch(params)      # data extraction on the SerpApi backend
        results = search.get_dict()         # JSON -> Python dict

        return results.get('filters', [])


    def save_to_json(self, data) -> None:
        with open(f'home_depot_search_{self.query.lower().replace(" ", "_")}.json', 'w',
                  encoding='utf-8') as json_file:
            json.dump(data, json_file, indent=2, ensure_ascii=False)


    @staticmethod
    def print(data) -> None:
        """Prints pretty JSON instead of regular print()"""
        print(json.dumps(data, indent=2, ensure_ascii=False))


class HomeDepotProduct(object):
    def __init__(self,
                 api_key: str,
                 product_id,
                 zip_code = None) -> None:
        self.api_key = api_key
        self.product_id = product_id
        self.zip_code = zip_code


    def __get_params(self) -> dict:
        params = {
            'api_key': self.api_key,        # https://serpapi.com/manage-api-key
            'engine': 'home_depot_product', # search engine
            'device': 'desktop',            # device to use to get the results
            'product_id': self.product_id,  # Home Depot ID of a product
        }

        # https://regex101.com/r/UnEKoe/1
        zip_pattern = re.compile(r'^\d{5}(-\d{4})?$')

        if self.zip_code:
            if not zip_pattern.match(self.zip_code):
                raise ValueError('Please set a correct ZIP code.')
            params['delivery_zip'] = self.zip_code

        return params


    def get_product(self) -> dict:
        """Returns:
            Dictionary with product data without reviews
        """
        params = self.__get_params()
        search = SerpApiSearch(params)      # data extraction on the SerpApi backend
        results = search.get_dict()         # JSON -> Python dict

        product_info = results['product_results']
        del product_info['upc']

        return product_info


    def save_to_json(self, data) -> None:
        with open(f'home_depot_product_{self.product_id}.json', 'w', encoding='utf-8') as json_file:
            json.dump(data, json_file, indent=2, ensure_ascii=False)


    @staticmethod
    def print(data) -> None:
        """Prints pretty JSON instead of regular print()"""
        print(json.dumps(data, indent=2, ensure_ascii=False))



languages = [
  {
    "code": "af",
    "name": "Afrikaans",
  },
  {
    "code": "ak",
    "name": "Akan",
  },
  {
    "code": "sq",
    "name": "Albanian",
  },
  {
    "code": "ws",
    "name": "Samoa",
  },
  {
    "code": "am",
    "name": "Amharic",
  },
  {
    "code": "ar",
    "name": "Arabic",
  },
  {
    "code": "hy",
    "name": "Armenian",
  },
  {
    "code": "az",
    "name": "Azerbaijani",
  },
  {
    "code": "eu",
    "name": "Basque",
  },
  {
    "code": "be",
    "name": "Belarusian",
  },
  {
    "code": "bem",
    "name": "Bemba",
  },
  {
    "code": "bn",
    "name": "Bengali",
  },
  {
    "code": "bh",
    "name": "Bihari",
  },
  {
    "code": "xx-bork",
    "name": "Bork, bork, bork!",
  },
  {
    "code": "bs",
    "name": "Bosnian",
  },
  {
    "code": "br",
    "name": "Breton",
  },
  {
    "code": "bg",
    "name": "Bulgarian",
  },
  {
    "code": "bt",
    "name": "Bhutanese",
  },
  {
    "code": "km",
    "name": "Cambodian",
  },
  {
    "code": "ca",
    "name": "Catalan",
  },
  {
    "code": "chr",
    "name": "Cherokee",
  },
  {
    "code": "ny",
    "name": "Chichewa",
  },
  {
    "code": "zh-cn",
    "name": "Chinese (Simplified)",
  },
  {
    "code": "zh-tw",
    "name": "Chinese (Traditional)",
  },
  {
    "code": "co",
    "name": "Corsican",
  },
  {
    "code": "hr",
    "name": "Croatian",
  },
  {
    "code": "cs",
    "name": "Czech",
  },
  {
    "code": "da",
    "name": "Danish",
  },
  {
    "code": "nl",
    "name": "Dutch",
  },
  {
    "code": "xx-elmer",
    "name": "Elmer Fudd",
  },
  {
    "code": "en",
    "name": "English",
  },
  {
    "code": "eo",
    "name": "Esperanto",
  },
  {
    "code": "et",
    "name": "Estonian",
  },
  {
    "code": "ee",
    "name": "Ewe",
  },
  {
    "code": "fo",
    "name": "Faroese",
  },
  {
    "code": "tl",
    "name": "Filipino",
  },
  {
    "code": "fi",
    "name": "Finnish",
  },
  {
    "code": "fr",
    "name": "French",
  },
  {
    "code": "fy",
    "name": "Frisian",
  },
  {
    "code": "gaa",
    "name": "Ga",
  },
  {
    "code": "gl",
    "name": "Galician",
  },
  {
    "code": "ka",
    "name": "Georgian",
  },
  {
    "code": "de",
    "name": "German",
  },
  {
    "code": "el",
    "name": "Greek",
  },
  {
    "code": "kl",
    "name": "Greenlandic",
  },
  {
    "code": "gn",
    "name": "Guarani",
  },
  {
    "code": "gu",
    "name": "Gujarati",
  },
  {
    "code": "xx-hacker",
    "name": "Hacker",
  },
  {
    "code": "ht",
    "name": "Haitian Creole",
  },
  {
    "code": "ha",
    "name": "Hausa",
  },
  {
    "code": "haw",
    "name": "Hawaiian",
  },
  {
    "code": "iw",
    "name": "Hebrew",
  },
  {
    "code": "hi",
    "name": "Hindi",
  },
  {
    "code": "hu",
    "name": "Hungarian",
  },
  {
    "code": "is",
    "name": "Icelandic",
  },
  {
    "code": "ig",
    "name": "Igbo",
  },
  {
    "code": "id",
    "name": "Indonesian",
  },
  {
    "code": "ia",
    "name": "Interlingua",
  },
  {
    "code": "ga",
    "name": "Irish",
  },
  {
    "code": "it",
    "name": "Italian",
  },
  {
    "code": "ja",
    "name": "Japanese",
  },
  {
    "code": "jw",
    "name": "Javanese",
  },
  {
    "code": "kn",
    "name": "Kannada",
  },
  {
    "code": "kk",
    "name": "Kazakh",
  },
  {
    "code": "rw",
    "name": "Kinyarwanda",
  },
  {
    "code": "rn",
    "name": "Kirundi",
  },
  {
    "code": "xx-klingon",
    "name": "Klingon",
  },
  {
    "code": "kg",
    "name": "Kongo",
  },
  {
    "code": "ko",
    "name": "Korean",
  },
  {
    "code": "kri",
    "name": "Krio (Sierra Leone)",
  },
  {
    "code": "ku",
    "name": "Kurdish",
  },
  {
    "code": "ckb",
    "name": "Kurdish (Soranî)",
  },
  {
    "code": "ky",
    "name": "Kyrgyz",
  },
  {
    "code": "lo",
    "name": "Laothian",
  },
  {
    "code": "la",
    "name": "Latin",
  },
  {
    "code": "lv",
    "name": "Latvian",
  },
  {
    "code": "ln",
    "name": "Lingala",
  },
  {
    "code": "lt",
    "name": "Lithuanian",
  },
  {
    "code": "loz",
    "name": "Lozi",
  },
  {
    "code": "lg",
    "name": "Luganda",
  },
  {
    "code": "ach",
    "name": "Luo",
  },
  {
    "code": "mk",
    "name": "Macedonian",
  },
  {
    "code": "mg",
    "name": "Malagasy",
  },
  {
    "code": "my",
    "name": "Malay",
  },
  {
    "code": "ml",
    "name": "Malayalam",
  },
  {
    "code": "mt",
    "name": "Maltese",
  },
  {
    "code": "mv",
    "name": "Maldives",
  },
  {
    "code": "mi",
    "name": "Maori",
  },
  {
    "code": "mr",
    "name": "Marathi",
  },
  {
    "code": "mfe",
    "name": "Mauritian Creole",
  },
  {
    "code": "mo",
    "name": "Moldavian",
  },
  {
    "code": "mn",
    "name": "Mongolian",
  },
  {
    "code": "sr-me",
    "name": "Montenegrin",
  },
  {
    "code": "ne",
    "name": "Nepali",
  },
  {
    "code": "pcm",
    "name": "Nigerian Pidgin",
  },
  {
    "code": "nso",
    "name": "Northern Sotho",
  },
  {
    "code": "no",
    "name": "Norwegian",
  },
  {
    "code": "nn",
    "name": "Norwegian (Nynorsk)",
  },
  {
    "code": "oc",
    "name": "Occitan",
  },
  {
    "code": "or",
    "name": "Oriya",
  },
  {
    "code": "om",
    "name": "Oromo",
  },
  {
    "code": "ps",
    "name": "Pashto",
  },
  {
    "code": "fa",
    "name": "Persian",
  },
  {
    "code": "xx-pirate",
    "name": "Pirate",
  },
  {
    "code": "pl",
    "name": "Polish",
  },
  {
    "code": "pt",
    "name": "Portuguese",
  },
  {
    "code": "pt-br",
    "name": "Portuguese (Brazil)",
  },
  {
    "code": "pt-pt",
    "name": "Portuguese (Portugal)",
  },
  {
    "code": "pa",
    "name": "Punjabi",
  },
  {
    "code": "qu",
    "name": "Quechua",
  },
  {
    "code": "ro",
    "name": "Romanian",
  },
  {
    "code": "rm",
    "name": "Romansh",
  },
  {
    "code": "nyn",
    "name": "Runyakitara",
  },
  {
    "code": "ru",
    "name": "Russian",
  },
  {
    "code": "gd",
    "name": "Scots Gaelic",
  },
  {
    "code": "sr",
    "name": "Serbian",
  },
  {
    "code": "sh",
    "name": "Serbo-Croatian",
  },
  {
    "code": "st",
    "name": "Sesotho",
  },
  {
    "code": "tn",
    "name": "Setswana",
  },
  {
    "code": "crs",
    "name": "Seychellois Creole",
  },
  {
    "code": "sn",
    "name": "Shona",
  },
  {
    "code": "sd",
    "name": "Sindhi",
  },
  {
    "code": "si",
    "name": "Sinhalese",
  },
  {
    "code": "sk",
    "name": "Slovak",
  },
  {
    "code": "sl",
    "name": "Slovenian",
  },
  {
    "code": "so",
    "name": "Somali",
  },
  {
    "code": "es",
    "name": "Spanish",
  },
  {
    "code": "es-419",
    "name": "Spanish (Latin American)",
  },
  {
    "code": "su",
    "name": "Sundanese",
  },
  {
    "code": "sw",
    "name": "Swahili",
  },
  {
    "code": "sv",
    "name": "Swedish",
  },
  {
    "code": "tg",
    "name": "Tajik",
  },
  {
    "code": "ta",
    "name": "Tamil",
  },
  {
    "code": "tt",
    "name": "Tatar",
  },
  {
    "code": "te",
    "name": "Telugu",
  },
  {
    "code": "th",
    "name": "Thai",
  },
  {
    "code": "ti",
    "name": "Tigrinya",
  },
  {
    "code": "to",
    "name": "Tonga",
  },
  {
    "code": "lua",
    "name": "Tshiluba",
  },
  {
    "code": "tum",
    "name": "Tumbuka",
  },
  {
    "code": "tr",
    "name": "Turkish",
  },
  {
    "code": "tk",
    "name": "Turkmen",
  },
  {
    "code": "tw",
    "name": "Twi",
  },
  {
    "code": "ug",
    "name": "Uighur",
  },
  {
    "code": "uk",
    "name": "Ukrainian",
  },
  {
    "code": "ur",
    "name": "Urdu",
  },
  {
    "code": "uz",
    "name": "Uzbek",
  },
  {
    "code": "vu",
    "name": "Vanuatu",
  },
  {
    "code": "vi",
    "name": "Vietnamese",
  },
  {
    "code": "cy",
    "name": "Welsh",
  },
  {
    "code": "wo",
    "name": "Wolof",
  },
  {
    "code": "xh",
    "name": "Xhosa",
  },
  {
    "code": "yi",
    "name": "Yiddish",
  },
  {
    "code": "yo",
    "name": "Yoruba",
  },
  {
    "code": "zu",
    "name": "Zulu",
  },
]


from .currencies import currencies
from .languages import languages
from selenium import webdriver
from selenium_stealth import stealth
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager
from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.keys import Keys
from selectolax.lexbor import LexborHTMLParser
import json, re, time


class AmazonParams(object):
    def __init__(self, currency: str, language: str) -> None:
        self.currency = currency
        self.language = language


    @staticmethod
    def get_currencies() -> list:
        return currencies


    @staticmethod
    def get_languages() -> list:
        return languages


    def _validate_params(self):
        selected_currency = next(
            (el['code'] for el in currencies if
             el['code'].lower() == self.currency.lower() or el['text'].lower() == self.currency.lower()),
            None
        )
        if self.currency and not selected_currency:
            raise ValueError(
                'Please select available currency (use "AmazonParams.get_currencies()" to get all currencies list).')

        selected_language = next(
            (el['code'] for el in languages if
             el['code'].lower() == self.language.lower() or el['text'].lower() == self.language.lower()),
            None
        )
        if self.language and not selected_language:
            raise ValueError(
                'Please select available language (use "AmazonParams.get_languages()" to get all languages list).')

        return selected_currency, selected_language


class AmazonSearch(AmazonParams):
    def __init__(self,
                 query: str,
                 results_limit: int = 10,
                 multiplier: float = 1.0,
                 currency: str = '',
                 language: str = '',
                 price_from: float = 0,
                 price_to: float = 0,
                 customer_reviews_rating: int = 0) -> None:
        super().__init__(currency, language)
        self.query = query
        self.results_limit = results_limit
        self.multiplier = multiplier
        self.price_from = price_from
        self.price_to = price_to
        self.customer_reviews_rating = customer_reviews_rating
        self.currency, self.language = self._validate_params()
        self.driver = self.__get_browser_instance()


    def __del__(self):
        self.driver.quit()


    def __get_browser_instance(self):
        service = Service(ChromeDriverManager().install())

        options = webdriver.ChromeOptions()
        options.add_argument('start-maximized')
        options.add_argument('--headless')
        options.add_argument('--no-sandbox')
        options.add_experimental_option('excludeSwitches', ['enable-automation', 'enable-logging'])
        options.add_experimental_option('useAutomationExtension', False)

        driver = webdriver.Chrome(service=service, options=options)

        stealth(
            driver,
            user_agent='Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36',
            languages=['en-US', 'en'],
            vendor='Google Inc.',
            platform='Win32',
            webgl_vendor='Intel Inc.',
            renderer='Intel Iris OpenGL Engine',
            fix_hairline=True,
        )

        url = f'https://www.amazon.com/s?k={self.query}'

        if self.language:
            url += f'&language={self.language}'

        if self.price_from != 0 or self.price_to != 0:
            if self.price_from > self.price_to or self.price_from < 0 or self.price_to < 0:
                raise ValueError('Please set correct price arguments.')
            url += f'&rh=p_36:{self.price_from}00-{self.price_to or 0}00'

        rating_number = int(self.customer_reviews_rating)

        if rating_number:
            if not rating_number or rating_number < 1 or rating_number > 4:
                raise Exception('Please set "customer_reviews_rating" argument from 1 to 4.')

        driver.get(url)
        WebDriverWait(driver, 10000).until(
            EC.visibility_of_element_located((By.CSS_SELECTOR, '.s-result-item:not(.a-section, .s-widget)'))
        )

        if self.currency:
            try:
                driver.find_element(By.CSS_SELECTOR, '#icp-touch-link-cop').click()
                time.sleep(self.multiplier * 2)
                driver.find_element(By.CSS_SELECTOR, '#icp-currency-dropdown-selected-item-prompt').click()
                time.sleep(self.multiplier * 2)
                driver.find_element(By.CSS_SELECTOR, f'#{self.currency} a').click()
                time.sleep(self.multiplier * 2)
                driver.find_element(By.CSS_SELECTOR, '#icp-save-button').click()
                time.sleep(self.multiplier * 2)
                WebDriverWait(driver, 10000).until(
                    EC.visibility_of_element_located((By.CSS_SELECTOR, '#reviewsRefinements'))
                )
            except:
                print('Error in currency')

        if self.customer_reviews_rating:
            selector_number = None

            if rating_number == 1:
                selector_number = 4
            elif rating_number == 2:
                selector_number = 3
            elif rating_number == 3:
                selector_number = 2
            elif rating_number == 4:
                selector_number = 1

            try:
                driver.find_element(By.CSS_SELECTOR, f'#reviewsRefinements ul > span:nth-child({selector_number})').click()
                time.sleep(self.multiplier * 1)
                WebDriverWait(driver, 10000).until(
                    EC.visibility_of_element_located((By.CSS_SELECTOR, '.s-result-item:not(.a-section, .s-widget)'))
                )
            except:
                print('Error in customer reviews rating')

        return driver

    def get_products(self) -> list:
        products = []

        try:
            while True:
                time.sleep(self.multiplier * 1)

                parser = LexborHTMLParser(self.driver.page_source)

                for product in parser.root.css('.s-result-item:not(.a-section, .s-widget)'):
                    title = product.css_first('h2').text(strip=True)
                    raw_subtitle = product.css_first('h2 + div')
                    subtitle = raw_subtitle.text(strip=True).replace('·', ' ') if raw_subtitle else None
                    raw_link = product.css_first('h2 a').attributes.get('href')
                    link = f'https://www.amazon.com{raw_link}'
                    raw_thumbnail = product.css_first('[data-component-type="s-product-image"] img')
                    thumbnail_srcset = raw_thumbnail.attributes.get('srcset') if raw_thumbnail else None
                    thumbnail = thumbnail_srcset.split(' ')[-2] if thumbnail_srcset else raw_thumbnail.attributes.get('src')
                    raw_badge = product.css_first('[data-component-type="s-status-badge-component"] .a-badge-text')
                    badge = raw_badge.text(strip=True) if raw_badge else None
                    raw_rating = product.css_first('.s-product-image-container + div > div:nth-child(2) span:first-child, .s-list-col-right > div > div > div:nth-child(2) > div > span:first-child')
                    raw_rating_attribute = raw_rating.attributes.get('aria-label') if raw_rating else None
                    rating = float(raw_rating_attribute.split(" ")[0]) if raw_rating_attribute else None
                    raw_reviews = product.css_first('.s-product-image-container + div > div:nth-child(2) > div > span:last-child, .s-list-col-right > div > div > div:nth-child(2) > div > span:last-child')
                    reviews = int(raw_reviews.text(strip=True).replace(',', '').replace('.', '')) if raw_reviews else None
                    raw_listing_history = product.css_first('.s-product-image-container + div > div:nth-child(2) > div:nth-child(2), .s-list-col-right > div > div > div:nth-child(2) > div:nth-child(2) > span')
                    listing_history = raw_listing_history.text(strip=True) if raw_listing_history else None
                    raw_options = product.css_first('.a-truncate-cut')
                    options = raw_options.text(strip=True) if raw_options else None
                    raw_additional_info = product.css_first('.s-product-image-container + div > div:nth-child(4) span.a-size-base, .s-list-col-right > div > div > div:nth-child(3) .a-section:nth-child(2)')
                    additional_info = raw_additional_info.text(strip=True) if raw_additional_info else None
                    raw_price = product.css_first('.s-product-image-container + div > div:nth-child(3) .a-price .a-offscreen, .s-list-col-right > div > div > div:nth-child(3) a .a-price .a-offscreen')
                    price = raw_price.text(strip=True) if raw_price else None

                    products.append({
                        'title': title,
                        'subtitle': subtitle,
                        'link': link,
                        'thumbnail': thumbnail,
                        'badge': badge,
                        'rating': rating,
                        'reviews': reviews,
                        'listing_history': listing_history,
                        'options': options,
                        'additional_info': additional_info,
                        'price': price,
                    })

                if len(products) >= self.results_limit:
                    products = products[:self.results_limit]
                    break

                try:
                    self.driver.find_element(By.CSS_SELECTOR, '.s-pagination-next:not(.s-pagination-disabled)').click()
                    time.sleep(self.multiplier * 1)
                    WebDriverWait(self.driver, 10000).until(
                        EC.visibility_of_element_located((By.CSS_SELECTOR, '.s-result-item:not(.a-section, .s-widget)'))
                    )
                    time.sleep(self.multiplier * 3)
                except:
                    print('Error in pagination')

            for index, product in enumerate(products, start=0):
                products[index]['position'] = index + 1
        except:
            print('Error in getting the product')
        finally:
            return products


    def save_to_json(self, data) -> None:
        with open(f'amazon_search_{self.query.lower().replace(" ", "_")}.json', 'w',
                  encoding='utf-8') as json_file:
            json.dump(data, json_file, indent=2, ensure_ascii=False)


    @staticmethod
    def print(data) -> None:
        """Prints pretty JSON instead of regular print()"""
        print(json.dumps(data, indent=2, ensure_ascii=False))


class AmazonProduct(AmazonParams):
    def __init__(self,
                 link: str,
                 reviews_limit: int = 10,
                 multiplier: float = 1.0,
                 currency: str = '',
                 language: str = '',) -> None:
        super().__init__(currency, language)
        self.link = link
        self.reviews_limit = reviews_limit
        self.multiplier = multiplier
        self.currency, self.language = self._validate_params()
        self.driver = self.__get_browser_instance()
        self.product_name = self.__get_product_name()


    def __del__(self):
        self.driver.quit()


    def __get_product_name(self):
        # https://regex101.com/r/27NCcr/1
        product_name = re.search(r'amazon\.com/(.+)/dp', self.link).group(1)

        return product_name


    def __get_browser_instance(self):
        service = Service(ChromeDriverManager().install())

        options = webdriver.ChromeOptions()
        options.add_argument('start-maximized')
        options.add_argument('--headless')
        options.add_argument('--no-sandbox')
        options.add_experimental_option('excludeSwitches', ['enable-automation', 'enable-logging'])
        options.add_experimental_option('useAutomationExtension', False)

        driver = webdriver.Chrome(service=service, options=options)

        stealth(
            driver,
            user_agent='Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36',
            languages=['en-US', 'en'],
            vendor='Google Inc.',
            platform='Win32',
            webgl_vendor='Intel Inc.',
            renderer='Intel Iris OpenGL Engine',
            fix_hairline=True,
        )

        if self.language:
            driver.get(self.link + f'&language={self.language}')
        else:
            driver.get(self.link)

        WebDriverWait(driver, 10000).until(
            EC.visibility_of_element_located((By.CSS_SELECTOR, 'span#productTitle'))
        )

        if self.currency:
            try:
                driver.find_element(By.CSS_SELECTOR, '#icp-touch-link-cop').click()
                time.sleep(self.multiplier * 2)
                driver.find_element(By.CSS_SELECTOR, '#icp-currency-dropdown-selected-item-prompt').click()
                time.sleep(self.multiplier * 2)
                driver.find_element(By.CSS_SELECTOR, f'#{self.currency} a').click()
                time.sleep(self.multiplier * 2)
                driver.find_element(By.CSS_SELECTOR, '#icp-save-button').click()
                time.sleep(self.multiplier * 2)
                WebDriverWait(driver, 10000).until(
                    EC.visibility_of_element_located((By.CSS_SELECTOR, 'span#productTitle'))
                )
            except:
                print('Error in currency')

        return driver


    def get_product_info(self) -> dict:
        parser = LexborHTMLParser(self.driver.page_source)

        raw_title = parser.root.css_first('span#productTitle')
        title = raw_title.text(strip=True) if raw_title else None

        raw_badge = parser.root.css_first('#acBadge_feature_div span > span')
        badge = raw_badge.text(strip=True) if raw_badge else None

        overview = {}
        for element in parser.root.css('#productOverview_feature_div tr'):
            key = element.css_first('td:first-child').text(strip=True).lower().replace(' ', '_')
            value = element.css_first('td:last-child').text(strip=True)
            overview[key] = value

        features = [
            element.text(strip=True)
            for element in parser.css('#featurebullets_feature_div li')
        ] if parser.css_first('#featurebullets_feature_div li') else None

        information = {}
        for element in parser.root.css('#productDetails_feature_div tr, #detailBulletsWrapper_feature_div li'):
            raw_key = element.css_first('th') if element.css_first('th') else element.css_first('.a-text-bold')
            key = raw_key.text().split('\n')[0].strip().lower().replace(' ', '_') if raw_key else None
            raw_value = element.css_first('td') if element.css_first('td') else element.css_first('.a-list-item span:last-child')
            value = raw_value.text(strip=True) if raw_value else None

            if str(key) in 'best_sellers_rank':
                if element.css_first('td'):
                    value = [el.text(strip=True) for el in element.css('td span span')]
                else:
                    raw_value = element.css_first('span.a-list-item').text(strip=True).split('#')
                    value = ['#' + el for el in raw_value[1:len(raw_value)]]
            elif str(key) in 'customer_reviews':
                continue

            if key is not None:
                information[key] = value

        raw_description = parser.root.css_first("#productDescription span")
        description = raw_description.text(strip=True) if raw_description else None

        raw_rating = parser.root.css_first('span#acrPopover')
        rating = float(
            raw_rating.attributes.get('aria-label').split()[0]
            if raw_rating.attributes.get('aria-label') else
            raw_rating.attributes.get('title').split()[0]
        ) if raw_rating else None

        raw_reviews = parser.root.css_first('#acrCustomerReviewLink')
        reviews = int(raw_reviews.text(strip=True).split(' ')[0].replace(',', '').replace('.', '')) if raw_reviews else None

        data = {
            'title': title,
            'link': self.link,
            'badge': badge,
            'overview': overview,
            'features': features,
            'information': information,
            'description': description,
            'buying_options': [],
            'by_feature': {},
            'rating': rating,
            'reviews': reviews,
        }

        return data


    def get_by_feature(self) -> dict:
        by_feature = {}

        try:
            self.driver.find_element(By.CSS_SELECTOR, 'body').send_keys(Keys.END)
            time.sleep(self.multiplier * 1)

            for i in range(10):
                self.driver.find_element(By.CSS_SELECTOR, 'body').send_keys(Keys.PAGE_UP)
                time.sleep(self.multiplier * 0.5)

            parser = LexborHTMLParser(self.driver.page_source)

            for element in parser.root.css('#cr-summarization-attributes-list > div'):
                key = element.css_first('.a-col-left').text(strip=True).lower().replace(' ', '_')
                value = float(element.css_first('i > span').text(strip=True))
                by_feature[key] = value
        except:
            print('Error in by feature')
        finally:
            return by_feature


    def get_buying_options(self) -> list:
        buying_options = []

        try:
            self.driver.find_element(By.CSS_SELECTOR, '#buybox-see-all-buying-choices').click()
            time.sleep(self.multiplier * 2)

            parser = LexborHTMLParser(self.driver.page_source)

            for element in parser.root.css('#aod-offer-list > div'):
                raw_condition = element.css_first('#aod-offer-heading h5')
                condition = raw_condition.text(strip=True).replace('\n', '') if raw_condition else None
                raw_price = element.css_first('.a-price .a-offscreen')
                price = raw_price.text(strip=True) if raw_price else None
                delivery_block = element.css_first('#mir-layout-DELIVERY_BLOCK')
                delivery = {
                    'date': delivery_block.css_first('span > span').text(strip=True),
                    'price': delivery_block.css_first('span').text(strip=True).split()[0]
                } if delivery_block else None
                raw_ships_from = element.css_first('#aod-offer-shipsFrom .aod-ships-from-country')
                ships_from = raw_ships_from.text(strip=True) if raw_ships_from else None
                raw_seller = element.css_first('#aod-offer-soldBy a')
                seller = raw_seller.text(strip=True) if raw_seller else None
                seller_link = 'https://www.amazon.com' + raw_seller.attributes.get('href') if raw_seller else None
                raw_seller_rating = element.css_first('#aod-offer-seller-rating')
                seller_rating = raw_seller_rating.text(strip=True) if raw_seller_rating else None
                sold_by = {
                    'seller': seller,
                    'seller_link': seller_link,
                    'seller_rating': seller_rating
                } if raw_seller else None

                buying_options.append({
                    'condition': condition,
                    'price': price,
                    'delivery': delivery,
                    'shipsFrom': ships_from,
                    'soldBy': sold_by
                })

            self.driver.find_element(By.CSS_SELECTOR, '#aod-close').click()
            time.sleep(self.multiplier * 1)
        except:
            print('Error in buying options')
        finally:
            return buying_options


    def get_product_reviews(self) -> list:
        reviews = []

        try:
            self.driver.execute_script('document.querySelector(\'[data-hook="see-all-reviews-link-foot"]\').click()')
            time.sleep(self.multiplier * 1)

            while True:
                self.driver.find_element(By.CSS_SELECTOR, 'body').send_keys(Keys.END)
                time.sleep(self.multiplier * 1)

                parser = LexborHTMLParser(self.driver.page_source)

                for element in parser.css('[data-hook="review"]'):
                    raw_name = element.css_first('.a-profile-name')
                    name = raw_name.text(strip=True) if raw_name else None
                    raw_avatar = element.css_first('.a-profile-avatar > img')
                    avatar = raw_avatar.attributes.get('src') if raw_avatar else None
                    raw_rating = element.css_first('i > span')
                    rating = float(raw_rating.text(strip=True).split(' ')[0]) if raw_rating else None
                    raw_heading = element.css_first('[data-hook="review-title"]')
                    heading = raw_heading.text(strip=True) if raw_heading else None
                    raw_date = element.css_first('[data-hook="review-date"]')
                    date = raw_date.text(strip=True).split(' on ')[-1] if raw_date else None
                    raw_badge = element.css_first('[data-hook="avp-badge"]')
                    badge = raw_badge.text(strip=True) if raw_badge else None
                    raw_review = element.css_first('[data-hook="review-body"]')
                    review = raw_review.text(strip=True) if raw_review else None

                    reviews.append({
                        'name': name,
                        'avatar': avatar,
                        'rating': rating,
                        'heading': heading,
                        'date': date,
                        'badge': badge,
                        'review': review
                    })

                if len(reviews) >= self.reviews_limit:
                    reviews = reviews[:self.reviews_limit]
                    break

                try:
                    self.driver.find_element(By.CSS_SELECTOR, '.a-pagination > li:last-child:not(.a-disabled)').click()
                    time.sleep(self.multiplier * 1)
                    self.driver.find_element(By.CSS_SELECTOR, '[data-hook="review"]')
                except:
                    break

            self.driver.find_element(By.CSS_SELECTOR, '#cm_cr-footer_dp_link .a-link-normal').click()
            time.sleep(self.multiplier * 1)

            for index, review in enumerate(reviews, start=0):
                review['position'] = index + 1
        except:
            print('Error in product reviews')
        finally:
            return reviews


    def get_product(self) -> dict:
        product = self.get_product_info()
        product['by_feature'] = self.get_by_feature()
        product['buying_options'] = self.get_buying_options()

        reviews = self.get_product_reviews()

        product_data = {
            'product': product,
            'reviews': reviews
        }

        return product_data


    def save_to_json(self, data) -> None:
        with open(f'amazon_product_{self.product_name.lower().replace("-", "_")}.json', 'w', encoding='utf-8') as json_file:
            json.dump(data, json_file, indent=2, ensure_ascii=False)


    @staticmethod
    def print(data) -> None:
        """Prints pretty JSON instead of regular print()"""
        print(json.dumps(data, indent=2, ensure_ascii=False))



from .domains import domains
from .countries import countries
from .languages import languages
from serpapi import GoogleSearch
from copy import deepcopy
from urllib.parse import urlsplit, parse_qsl
import json


class GoogleShoppingParams(object):
    def __init__(self, domain: str, country: str, language: str) -> None:
        self.domain = domain
        self.country = country
        self.language = language


    @staticmethod
    def get_domains() -> list:
        return domains


    @staticmethod
    def get_countries() -> list:
        return countries


    @staticmethod
    def get_languages() -> list:
        return languages


    def _validate_params(self):
        selected_domain = next(
            (el for el in domains if
             el['domain'].lower() == self.domain.lower() or el['country_name'].lower() == self.domain.lower()),
            None
        )
        if self.domain and not selected_domain:
            raise ValueError(
                'Please select available domain (use "GoogleShoppingParams.get_domains()" to get all domains list).')

        selected_country = next(
            (el['code'] for el in countries if
             el['code'].lower() == self.country.lower() or el['name'].lower() == self.country.lower()),
            None
        )
        if self.country and not selected_country:
            raise ValueError(
                'Please select available country (use "GoogleShoppingParams.get_countries()" to get all countries list).')

        selected_language = next(
            (el['code'] for el in languages if
             el['code'].lower() == self.language.lower() or el['name'].lower() == self.language.lower()),
            None
        )
        if self.language and not selected_language:
            raise ValueError(
                'Please select available language (use "GoogleShoppingParams.get_languages()" to get all languages list).')

        domain = selected_domain['domain'] if selected_domain else 'google.com'
        country = selected_country or selected_domain['default_country_code'] if selected_domain else 'us'
        language = selected_language or selected_domain['default_language_code'] if selected_domain else 'en'

        return domain, country, language


class GoogleShoppingSearch(GoogleShoppingParams):
    def __init__(self,
                 api_key: str,
                 query: str,
                 price_from: float = 0,
                 price_to: float = 0,
                 results_limit: int = 10,
                 sort: int = 1,
                 domain: str = 'google.com',
                 country: str = 'us',
                 language: str = 'en') -> None:
        self.api_key = api_key
        self.query = query
        self.price_from = price_from
        self.price_to = price_to
        self.results_limit = results_limit
        self.sort = sort
        super().__init__(domain, country, language)


    def __get_tbs(self) -> str:
        tbs = ''

        if self.sort == 1:
            tbs = 'p_ord:r'
        elif self.sort == 2:
            tbs = 'p_ord:rv'
        elif self.sort == 3:
            tbs = 'p_ord:p'
        elif self.sort == 4:
            tbs = 'p_ord:pd'
        else:
            raise ValueError('Please set correct sort argument (from 1 to 4).')

        if self.price_from != 0 or self.price_to != 0:
            if self.price_from > self.price_to or self.price_from < 0 or self.price_to < 0:
                raise ValueError('Please set correct price arguments.')
            tbs += f",mr:1,price:1,ppr_min:{self.price_from},ppr_max:{self.price_to}"

        return tbs


    def __get_params(self) -> dict:
        domain, country, language = self._validate_params()

        params = {
            'api_key': self.api_key,        # https://serpapi.com/manage-api-key
            'engine': 'google_shopping',    # search engine
            'device': 'desktop',            # device to use to get the results
            'q': self.query,                # search query
            'google_domain': domain,        # Google domain to use
            'gl': country,                  # country of the search
            'hl': language,                 # language of the search
            'start': 0,                     # result offset
            'num': 100,                     # maximum number of results
            'tbs': self.__get_tbs()         # advanced search parameters
        }

        return params


    def get_products(self) -> list:
        params = self.__get_params()        # parameters for search
        search = GoogleSearch(params)       # data extraction on the SerpApi backend
        results = search.get_dict()         # JSON -> Python dict

        products = []
        limit = deepcopy(self.results_limit)

        # pagination if needed
        while True:
            shopping_results = results.get('shopping_results', [])

            if limit < 100:
                products.extend(shopping_results[:limit])
                break

            limit -= 100
            products.extend(shopping_results)

            params['start'] += params['num']
            results = search.get_dict()

        # index recalculation because on each page the position starts from 1
        for index, product in enumerate(products, start=0):
            products[index]['position'] = index + 1

            del products[index]['serpapi_product_api']
            del products[index]['number_of_comparisons']
            del products[index]['comparison_link']
            del products[index]['serpapi_product_api_comparisons']

        return products


    def get_filters(self) -> list:
        params = self.__get_params()        # parameters for search
        search = GoogleSearch(params)       # data extraction on the SerpApi backend
        results = search.get_dict()         # JSON -> Python dict

        return results.get('filters', [])


    def save_to_json(self, data) -> None:
        with open(f'google_shopping_search_{self.query.lower().replace(" ", "_")}.json', 'w',
                  encoding='utf-8') as json_file:
            json.dump(data, json_file, indent=2, ensure_ascii=False)


    @staticmethod
    def print(data) -> None:
        """Prints pretty JSON instead of regular print()"""
        print(json.dumps(data, indent=2, ensure_ascii=False))


class GoogleShoppingProduct(GoogleShoppingParams):
    def __init__(self,
                 api_key: str,
                 product_id,
                 reviews_limit: int = 10,
                 domain: str = 'google.com',
                 country: str = 'us',
                 language: str = 'en') -> None:
        self.api_key = api_key
        self.product_id = product_id
        self.reviews_limit = reviews_limit
        super().__init__(domain, country, language)


    def __get_params(self) -> dict:
        domain, country, language = self._validate_params()

        params = {
            'api_key': self.api_key,        # https://serpapi.com/manage-api-key
            'engine': 'google_product',     # search engine
            'device': 'desktop',            # device to use to get the results
            'product_id': self.product_id,  # Google Shopping ID of a product
            'google_domain': domain,        # Google domain to use
            'gl': country,                  # country of the search
            'hl': language,                 # language of the search
        }

        return params


    def get_product_info(self) -> dict:
        params = self.__get_params()        # parameters for search
        search = GoogleSearch(params)       # data extraction on the SerpApi backend
        results = search.get_dict()         # JSON -> Python dict

        return results['product_results']


    def get_product_reviews(self) -> list:
        params = self.__get_params()        # parameters for search
        params['reviews'] = True            # more reviews, could be also set as '1` which is the same as True
        search = GoogleSearch(params)       # data extraction on the SerpApi backend
        results = search.get_dict()         # JSON -> Python dict

        reviews = []

        product_reviews = results.get('product_results', {}).get('reviews', 0)

        if product_reviews:
            limit = self.reviews_limit if self.reviews_limit < product_reviews else product_reviews

            while True:
                reviews_results = results.get('reviews_results', {}).get('reviews', [])

                if limit < 10:
                    reviews.extend(reviews_results[:limit])
                    break

                limit -= 10
                reviews.extend(reviews_results)

                if 'next' in results.get('serpapi_pagination', {}):
                    search.params_dict.update(dict(parse_qsl(urlsplit(results.get('serpapi_pagination', {}).get('next')).query)))
                else:
                    break

                results = search.get_dict() # new page results

        # index recalculation because on each page the review position starts from 1
        for index, review in enumerate(reviews, start=0):
            review['position'] = index + 1

        return reviews


    def get_product(self) -> dict:
        product_data = {
            'product': self.get_product_info(),
            'reviews': self.get_product_reviews()
        }

        return product_data


    def save_to_json(self, data) -> None:
        with open(f'google_shopping_product_{self.product_id}.json', 'w',
                  encoding='utf-8') as json_file:
            json.dump(data, json_file, indent=2, ensure_ascii=False)


    @staticmethod
    def print(data) -> None:
        """Prints pretty JSON instead of regular print()"""
        print(json.dumps(data, indent=2, ensure_ascii=False))



languages = [
  { 'text': 'English', 'code': 'en_US' },
  { 'text': 'español', 'code': 'es_US' },
  { 'text': 'العربية', 'code': 'ar_AE' },
  { 'text': 'Deutsch', 'code': 'de_DE' },
  { 'text': 'עברית', 'code': 'he_IL' },
  { 'text': '한국어', 'code': 'ko_KR' },
  { 'text': 'português', 'code': 'pt_BR' },
  { 'text': '中文 (简体)', 'code': 'zh_CN' },
  { 'text': '中文 (繁體)', 'code': 'zh_TW' },
]


domains = [
  { 'domain': 'ebay.com.au', 'country': 'Australia' },
  { 'domain': 'ebay.at', 'country': 'Austria' },
  { 'domain': 'ebay.ca', 'country': 'Canada' },
  { 'domain': 'ebay.fr', 'country': 'France' },
  { 'domain': 'ebay.de', 'country': 'Germany' },
  { 'domain': 'ebay.com.hk', 'country': 'Hong Kong' },
  { 'domain': 'ebay.ie', 'country': 'Ireland' },
  { 'domain': 'ebay.it', 'country': 'Italy' },
  { 'domain': 'ebay.com.my', 'country': 'Malaysia' },
  { 'domain': 'ebay.nl', 'country': 'Netherlands' },
  { 'domain': 'ebay.ph', 'country': 'Philippines' },
  { 'domain': 'ebay.pl', 'country': 'Poland' },
  { 'domain': 'ebay.com.sg', 'country': 'Singapore' },
  { 'domain': 'ebay.es', 'country': 'Spain' },
  { 'domain': 'ebay.ch', 'country': 'Switzerland' },
  { 'domain': 'ebay.co.uk', 'country': 'United Kingdom' },
  { 'domain': 'ebay.com', 'country': 'United States' },
]


from setuptools import setup, find_packages

README = ''
with open('README.md', 'r', encoding='utf-8') as readme_file:
    README = readme_file.read()

setup(
    name='ecommerce-scraper-py',
    description = 'Scrape ecommerce websites such as Amazon, eBay, Walmart, Home Depot, Google Shopping from a single module in Python🐍',
    url='https://github.com/dimitryzub/ecommerce-scraper-py',
    version='0.1.0',
    license='MIT',
    author='Artur Chukhrai',
    author_email='chukhraiartur@gmail.com',
    maintainer='Artur Chukhrai, Dmitiry Zub',
    maintainer_email='chukhraiartur@gmail.com, dimitryzub@gmail.com',
    long_description_content_type='text/markdown',
    long_description=README,
    include_package_data=True,
    python_requires='>=3.8',
    packages=find_packages(),
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS',
        'Operating System :: POSIX :: Linux',
        'Topic :: Internet',
        'Natural Language :: English',
        'Topic :: Utilities',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        "License :: OSI Approved :: MIT License"
    ],
    keywords=[
        'amazon',
        'ebay',
        'home depot',
        'walmart',
        'google shopping',
        'serpapi',
        'scraper',
        'python',
        'ecommerce',
        'e-commerce',
        'web scraping',
        'python web scraping',
        'google-search-results'
    ],
    install_requires=[
        'google-search-results>=2.4',
        'selenium>=4.3',
        'selectolax>=0.3',
        'webdriver-manager>=3.8',
        'selenium-stealth>=1.0'
    ],
    project_urls={
        'Documentation': 'https://github.com/dimitryzub/ecommerce-scraper-py',
        'Source': 'https://github.com/dimitryzub/ecommerce-scraper-py',
        'Tracker': 'https://github.com/dimitryzub/ecommerce-scraper-py/issues',
    }
)


currencies = [
  { 'text': 'US Dollar', 'code': 'USD' },
  { 'text': 'Chilean Peso', 'code': 'CLP' },
  { 'text': 'Colombian Peso', 'code': 'COP' },
  { 'text': 'Hong Kong Dollar', 'code': 'HKD' },
  { 'text': 'Indonesian Rupiah', 'code': 'IDR' },
  { 'text': 'Israeli New Shekel', 'code': 'ILS' },
  { 'text': 'South Korean Won', 'code': 'KRW' },
  { 'text': 'Malaysian Ringgit', 'code': 'MYR' },
  { 'text': 'New Zealand Dollar', 'code': 'NZD' },
  { 'text': 'Thai Baht', 'code': 'THB' },
  { 'text': 'New Taiwan Dollar', 'code': 'TWD' },
  { 'text': 'Costa Rican Colón', 'code': 'CRC' },
  { 'text': 'Peruvian Sol', 'code': 'PEN' },
  { 'text': 'Uruguayan Peso', 'code': 'UYU' },
  { 'text': 'Brazilian Real', 'code': 'BRL' },
  { 'text': 'Australian Dollar', 'code': 'AUD' },
  { 'text': 'Canadian Dollar', 'code': 'CAD' },
  { 'text': 'Chinese Yuan', 'code': 'CNY' },
  { 'text': 'Euro', 'code': 'EUR' },
  { 'text': 'Japanese Yen', 'code': 'JPY' },
  { 'text': 'Mexican Peso', 'code': 'MXN' },
  { 'text': 'British Pound', 'code': 'GBP' },
  { 'text': 'United Arab Emirates Dirham', 'code': 'AED' },
  { 'text': 'Singapore Dollar', 'code': 'SGD' },
  { 'text': 'Saudi Riyal', 'code': 'SAR' },
  { 'text': 'Norwegian Krone', 'code': 'NOK' },
  { 'text': 'Argentine Peso', 'code': 'ARS' },
  { 'text': 'Armenian Dram', 'code': 'AMD' },
  { 'text': 'Aruban Florin', 'code': 'AWG' },
  { 'text': 'Azerbaijani Manat', 'code': 'AZN' },
  { 'text': 'Bahamian Dollar', 'code': 'BSD' },
  { 'text': 'Belize Dollar', 'code': 'BZD' },
  { 'text': 'Bolivian Boliviano', 'code': 'BOB' },
  { 'text': 'Brunei Dollar', 'code': 'BND' },
  { 'text': 'Bulgarian Lev', 'code': 'BGN' },
  { 'text': 'Cambodian Riel', 'code': 'KHR' },
  { 'text': 'Cayman Islands Dollar', 'code': 'KYD' },
  { 'text': 'Dominican Peso', 'code': 'DOP' },
  { 'text': 'East Caribbean Dollar', 'code': 'XCD' },
  { 'text': 'Egyptian Pound', 'code': 'EGP' },
  { 'text': 'Ghanaian Cedi', 'code': 'GHS' },
  { 'text': 'Guatemalan Quetzal', 'code': 'GTQ' },
  { 'text': 'Hungarian Forint', 'code': 'HUF' },
  { 'text': 'Indian Rupee', 'code': 'INR' },
  { 'text': 'Jamaican Dollar', 'code': 'JMD' },
  { 'text': 'Kazakhstani Tenge', 'code': 'KZT' },
  { 'text': 'Kenyan Shilling', 'code': 'KES' },
  { 'text': 'Lebanese Pound', 'code': 'LBP' },
  { 'text': 'Honduran Lempira', 'code': 'HNL' },
  { 'text': 'Macanese Pataca', 'code': 'MOP' },
  { 'text': 'Mauritian Rupee', 'code': 'MUR' },
  { 'text': 'Moroccan Dirham', 'code': 'MAD' },
  { 'text': 'Namibian Dollar', 'code': 'NAD' },
  { 'text': 'Nigerian Naira', 'code': 'NGN' },
  { 'text': 'Panamanian Balboa', 'code': 'PAB' },
  { 'text': 'Paraguayan Guarani', 'code': 'PYG' },
  { 'text': 'Qatari Riyal', 'code': 'QAR' },
  { 'text': 'Russian Ruble', 'code': 'RUB' },
  { 'text': 'South African Rand', 'code': 'ZAR' },
  { 'text': 'Tanzanian Shilling', 'code': 'TZS' },
  { 'text': 'Philippine Peso', 'code': 'PHP' },
  { 'text': 'Trinidad & Tobago Dollar', 'code': 'TTD' },
  { 'text': 'Mongolian Tugrik', 'code': 'MNT' },
  { 'text': 'Turkish Lira', 'code': 'TRY' },
  { 'text': 'Vietnamese Dong', 'code': 'VND' },
  { 'text': 'Swedish Krona', 'code': 'SEK' },
  { 'text': 'Polish Zloty', 'code': 'PLN' },
]